c Find best fit SED and expected magnitudes at a fixed z
c Works with the same SEDs than hyperz.
c Input: photometric catalogue with zspec (or zphot) in a column after the
c        photometric catalogue columns
c Output: file format is id, zspec, chi2,P,itype,iage,age,av,norm,
c        (mags(i) i=1,nfilt),m_abs
c modif July 25, 2006: added parameter SN_LIMIT to check the S/N in each 
c       filter; if S/N<SN_LIMIT then mag=99 (non detection)
c modif February 10, 2007: changed conversion magAB-> fluxes used in the fit 
c       and used consistent conversion in input and output 
c modif October 02, 2007: changed frequency integrals (too much c/nu^2 factors)
c modif April 07, 2010: revised input/output units 
c       OUTPUT_TYPE=0 [erg/cm2/s/A]
c       OUTPUT_TYPE=1 [muJy]
c       OUTPUT_TYPE=2 [mag AB]
c       checked for input in AB magnitudes (MAG_TYPE=1)
c modif October 28, 2010: added last column in output file with D_L [Mpc]
c modif October 29, 2010: added in *.out_zspec the input info on maximum 
c       wavelength and law_err
c modif June 28, 2012: added hidden parameter iband for the 
c       bandpass response function (energy vs photons)
c modif July 30, 2012: working option of Pegase SEDs, added parameter 
c       TRANS_ABS=type of transmission functions for FILT_M_ABS;;
c       added ERR_MIN_LAW=2 as in hyperzmass;
c       added WL_MIN and UV_FILTERS parameters;
c       check of Vega SED: removed scaling factor, the provided sed must 
c       be in units erg/s/cm2/A
c modif August 08, 2012: changed method to compute AB magnitudes, using
c       only integrals as a function of lambda: 
c       int(f(nu)*R(nu) dnu)/int(R(nu) dnu) = 
c       int(f(lambda)*R(lambda) dlambda)/[c*int(R(lambda)/lambda^2 dlambda]
c       replacing f(nu) dnu = f(lambda) dlambda, 
c       R(nu)=R(lambda), dnu = (c/lambda^2) dlambda
c modif September 04, 2012: changed method of reading and storing filters'  
c       transmission functions: removed convol_new, compos_new, filter0 
c       and filter.dec, englobed in a single routine read_filt 
c
c    Version: 05/09/2012
      program hyper_zspec
      INTEGER i,ii,iii,j,k,ks,n_files,ncomp,kfil,iw_vega,iw,n_filt,iob,
     .     nob,id,iout_s,iout_m,law_rd,law_err,nfil,iroot,matrix,iw_ob,
     .     i_typ,i_red,i_age,itype,ntyp_bc,ntyp_as,ntyp_pg,i_bc,i_as,
     .     i_pg,iage,nlyf,iwav,last,nred,ired,law_nd,i_lyman,it_pg,
     .     mxage,mxage_bc,mxage_pg,mxtyp,mxtyp_bc,mxtyp_as,mxtyp_pg,
     .     mxfil,mxred,mxlyf,mxwl,mxwl_bc,mxwl_as,mxwl_pg,ndeg,magtyp,
     .     iunit_sed,iblank,input,nparam,iagemax,iagemin,np,mxwf,
     .     nfilt_mabs,n_fil_w,n_head,nemline,ncolzs,icheck,imabs,irebin,
     .     ir_filters,uv_filters,iage_check,iband,iband_abs,mxvega,ierr,
     .     mxrp,iseg,imf,jo,ic1,mxafil,ifilt
      REAL*8 zs,w_vega,f_vega,t_bc,t_pg,f_bc,f_as,f_pg,amag,amag1,err_p,
     .     mag_nd,err_nd,err_min,avmin,avmax,averr,wl,surf,c_corr,
     .     err_fl,zmax,zmin,zz,y,filter,av,age,fl,qq,qqq,w0,wmin1,wmax1,
     .     wmin0,wmax0,wsyn,ysyn,yob,vega,flux,conv_mag,bp1,time_tot,
     .     f_jy,err_jy,deg,pchi2,qchi2,zero,chi2,r_chi2,amabs_1,amabs_2,
     .     amagb,dif_w,dif_w_min,fl_temp,mag_temp,func,prob,err,ww,h0,
     .     om,ov,ageuni,tz,r,d_l,fl_nd,fl1,amagbbf,rlin,chi2min,c,f2,
     .     trapz1,fact_lyf,r_lyf,wlmax,wlmin,scl,flux0,amagfit,fluxfit,
     .     yy,rcol(100),mag_nd1,z_form,age_form,age_min,age_max,wl_ir,
     .     wl_uv,dl,dmod,sn,sn_lim,fluxn,fluxerrn,amgal,amstar,sfr,
     .     err_minf,xlam,rr2,tau,lsun
      REAL*8 fff,wt,sxx,sxy,b,bb,aa8
      REAL wv,fv,ff,w_bc,w_as,w_pg,t_bc0,cputime,rmag,rerr_p,aa,rzs
c
      PARAMETER (err=1.D-8,nparam=38,imf=10,lsun=3.826D33)
      PARAMETER (c=2.99792458D+18) ! speed of light in AA/s
      INCLUDE 'dimensionz.dec'
      PARAMETER (mxtyp=mxtyp_bc+mxtyp_as+mxtyp_pg)

      LOGICAL header
      REAL ml,mu,xx(imf),lm(imf),um(imf),baux(imf),cn(imf),cc(imf),totm,
     .     totn,avs,tau0,zsun
      CHARACTER header_line*5000
      CHARACTER filesed(mxtyp)*160,sed_abc(mxtyp)*2,line*240
      CHARACTER*160 spectra,spectra0,spectra1,spectra2,filterout,
     .     amatrix,filedef,filein,filt_file,cat_in,spec_vega,
     .     filtresfile,filered,line_sed,copyright1,copyright2
      CHARACTER csed1*1,csed2*1,cage*1,cout_m*1,cout_s*1,root*12,
     .     date*24,vv*5,line_pg*80,cmag*5,cout*5
      CHARACTER*250 fmt,fmt0,fmt1,fmt2,fmt3,fmt4,fmt00
      CHARACTER mags(mxfil)*15,dmags(mxfil)*15,ch(mxfil)*3,string_id*80
      CHARACTER*40 cname,param(nparam)
c
      DIMENSION input(nparam),kfil(10),ks(mxtyp)
      DIMENSION ntyp_bc(mxtyp),ntyp_as(mxtyp),ntyp_pg(mxtyp)
      DIMENSION w_bc(mxwl_bc,mxtyp_bc),f_bc(mxwl_bc,mxage_bc,mxtyp_bc)
      DIMENSION w_as(mxwl_as,mxtyp_as),f_as(mxwl_as,1,mxtyp_as)
      DIMENSION w_pg(mxwl_pg,mxtyp_pg),f_pg(mxwl_pg,mxage_pg,mxtyp_pg)
      DIMENSION tau(mxtyp),zsun(mxtyp)
      DIMENSION nfil(mxfil+1),iw(mxtyp),i_lyman(mxfil)
      DIMENSION t_bc0(mxage_bc),t_bc(mxage_bc),t_pg(mxage_pg),ff(mxwl)
      DIMENSION w_vega(mxvega),f_vega(mxvega),vega(mxfil+1)
      DIMENSION bp1(mxfil+1),conv_mag(mxfil+1),surf(mxfil+1),f2(mxfil+1)
      DIMENSION c_corr(mxfil),fl_nd(mxfil),fl1(mxfil),err_minf(mxfil)
      DIMENSION f_jy(mxfil),err_jy(mxfil),fl_temp(mxfil),mag_temp(mxfil)
      DIMENSION rmag(mxfil),rerr_p(mxfil),amagfit(mxfil)
      DIMENSION fl(mxfil),err_fl(mxfil)
      DIMENSION fluxfit(mxfil),amag(mxfil),amag1(mxfil),err_p(mxfil)
      DIMENSION mag_nd(mxfil),mag_nd1(mxfil),err_nd(mxfil),law_nd(mxfil)
      DIMENSION ysyn(mxwl),wsyn(mxwl),ww(mxwl),yob(mxwl),y(mxwl)
      DIMENSION wl(mxfil+1),flux(mxfil+1,mxage,mxtyp,mxred),yy(mxwl)
      DIMENSION fluxn(mxfil+1),fluxerrn(mxfil+1)
      DIMENSION xlam(mxwf),rr2(mxwf)
c
      EXTERNAL cputime,rlin,filter,pchi2,qchi2,zero,func,tz,trapz1,d_l
      COMMON/compose/ifilt,np(mxfil)
      COMMON/rfilter/r(mxwf,mxfil,2)
      COMMON/nfree/deg,prob
      COMMON/redfile/filered
      COMMON/filterres/filtresfile
      COMMON/lyf_sigma/fact_lyf
c_________________________________________________________________________

      vv='2.2'
      copyright1='    Hyperzspec code - Version '//vv
     .     //' (September 5th, 2012)' 
      copyright2=' by Micol Bolzonella'
      WRITE(*,*)
      WRITE(*,'(a80)') copyright1
      WRITE(*,*)
      WRITE(*,'(a80)') copyright2
      WRITE(*,*)

      iband=1          ! bandpass response function unit for observed filters: 
                       ! 0.energy; 1.photons (see e.g. Bessell 2005)
                       ! this is not a free parameter, if you need to change
                       ! it modify the value here and recompile
      iband_abs=1      ! bandpass response function unit for Mabs filter
      nlyf=1           ! considering only the mean Madau value
      r_lyf=1.D0
      fact_lyf=1.D0
      law_err=0
      imabs=1
      amabs_1=-100.0D0
      amabs_2=100.0D0
      sn_lim=-1.D0
      ir_filters=0
      uv_filters=0
      ndeg=1
      deg=DBLE(REAL(ndeg))

 11   filein='hyperzspec.param'
      last=INDEX(filein,'  ')-1
      WRITE(*,*) 'Name of parameter file'
      WRITE(*,*) '[default = ',filein(1:last),']'
      READ(*,'(A)') line
      IF (line.ne.' ') THEN
         READ(line,'(A)',ERR=11) filein
      END IF
      time_tot=DBLE(cputime())
c ------------------------------------------------------------------------
c                       Reading parameters file 
c ------------------------------------------------------------------------
c ..... Parameters' names
      param(1)='A0VSED'
      param(2)='FILTERS_RES'
      param(3)='FILTERS_FILE'
      param(4)='TEMPLATES_FILE'
      param(5)='MAG_TYPE'
      param(6)='ERR_MAG_MIN'
      param(7)='ERR_MIN_LAW'
      param(8)='REDDENING_LAW'
      param(9)='AV_MIN'
      param(10)='AV_MAX'
      param(11)='AV_STEP'
      param(12)='OUTPUT_FILE'
      param(13)='H0'
      param(14)='OMEGA_M'
      param(15)='OMEGA_V'
      param(16)='FILT_M_ABS'
      param(17)='CATALOG_FILE'
c      param(18)='AGE_CHECK' ! obsolete
      param(18)='AGE_TYPE'
      param(19)='M_ABS_MIN'
      param(20)='M_ABS_MAX'
      param(21)='NCOL_ZSPEC'
      param(22)='SED_OBS_FILE'
      param(23)='SED_TEMP_FILE'
      param(24)='MATRIX'
      param(25)='SPECTRUM'
      param(26)='OUTPUT_TYPE'
      param(27)='M_ABS_ALG'
      param(28)='TRANS_ABS'
      param(29)='WL_MAX'
      param(30)='IR_FILTERS'
      param(31)='WL_MIN'
      param(32)='UV_FILTERS'
      param(33)='Z_FORM'
      param(34)='AGE_MIN'
      param(35)='AGE_MAX'
      param(36)='AGE_REBIN'
      param(37)='SN_LIMIT'
      param(38)='FILE_EXT_LAW'

      DO i=1,nparam
         input(i)=0
      END DO

      OPEN(77,FILE=filein,STATUS='old',ERR=11)
      WRITE(*,*) 'Reading parameter file :'
 7    READ(77,'(A)',ERR=10,END=10) line
      DO i=1,80
         IF (line(i:i).ne.' ') THEN
            line=line(i:)
            GO TO 8
         END IF
      END DO
      GO TO 7  ! blank line
 8    IF (line(1:1).eq.'#') GO TO 7 ! comment line
      iblank=INDEX(line,' ')
      cname=line(1:iblank-1) ! variable name
      line=line(iblank:)
      DO i=1,80
         IF (line(i:i).ne.' ') THEN
            line=line(i:)
            IF (line(1:1).eq.'#') THEN
               WRITE(*,*) 'Missed value of variable ', cname
               STOP
            END IF
            GO TO 9
         END IF
      END DO
      WRITE(*,*) 'Missed value of variable ', cname
      STOP
 9    iblank=INDEX(line,' ')
      line=line(1:iblank-1) ! part of string containing value of variable
c ..... File containing Vega spectrum
      IF (cname.eq.param(1)) THEN
         READ(line,'(A)') spec_vega
         input(1)=1
         GO TO 7
c ..... File FILTER.RES
      ELSE IF (cname.eq.param(2)) THEN
         READ(line,'(A)') filtresfile
         input(2)=1
         GO TO 7
c ..... File containing filter parameters and instructions for 
c       non detected objects (nd) or out of file (oof) for each filter
c       0 -> flux = -1 (filter not considered) for either nd or oof
c       1 -> flux = -1 for oof, flux = 0, err_flux = flux_nd for nd
c       2 -> flux = -1 for oof, flux = flux_nd/2, err_flux = flux_nd/2 for nd
c       3 -> flux = -1 for oof, mag = mag_nd,  err = err_nd for nd
c       4 -> flux = -1 for oof, reject solutions with flux > flux_nd
      ELSE IF (cname.eq.param(3)) THEN
         READ(line,'(A)') filt_file
         input(3)=1
         GO TO 7
c ..... File containing synthetic spectra filenames
      ELSE IF (cname.eq.param(4)) THEN
         READ(line,'(A)') filedef
         input(4)=1
         GO TO 7
c ..... Magnitude type:
c       0 -> standard magnitudes
c       1 -> AB magnitudes
c       2 -> Flux in microJansky
      ELSE IF (cname.eq.param(5)) THEN
         READ(line,*) magtyp
         IF (magtyp.ne.0.and.magtyp.ne.1.and.magtyp.ne.2) THEN
            WRITE(*,*) 'MAG_TYPE must be 0 , 1 or 2'
            STOP
         END IF
         IF (magtyp.eq.0) THEN
            cmag='Vega'
         ELSE IF (magtyp.eq.1) THEN
            cmag='AB'
         ELSE IF (magtyp.eq.2) THEN
            cmag='muJy'
         END IF
         input(5)=1
         GO TO 7
c ..... Minimum error in magnitude 
      ELSE IF (cname.eq.param(6)) THEN
         READ(line,*) err_min
         input(6)=1
         GO TO 7
c ..... Photometric errors:
c       0 -> if error < err_min => err = err_min
c       1 -> err_min added in quadrature to err in catalogue
c       2 -> err_minf(i) read from filters' file added in quadrature 
c            to err in catalogue, err_min is also used as minimum error
      ELSE IF (cname.eq.param(7)) THEN
         READ(line,*) law_err
         IF (law_err.ne.0.and.law_err.ne.1.and.law_err.ne.2) THEN
            WRITE(*,*) 'ERR_MIN_LAW must be 0, 1, or 2'
            STOP
         END IF
         input(7)=1
         GO TO 7
c ..... Reddening:
c       0 -> No reddening
c       1 -> Allen, 1976, standard MW
c       2 -> Seaton, 1979, standard MW
c       3 -> Fitzpatrick, 1986, LMC
c       4 -> Prevot, 1984 + Bouchet, 1985, SMC
c       5 -> Calzetti et al., 2000
      ELSE IF (cname.eq.param(8)) THEN
         READ(line,*) law_rd
         IF (law_rd.ne.0.and.law_rd.ne.1.and.law_rd.ne.2.and.
     .        law_rd.ne.3.and.law_rd.ne.4.and.law_rd.ne.5.and.
     .        law_rd.ne.-1) THEN
            WRITE(*,*) 'REDDENING_LAW must be -1 -- 5'
            STOP
         END IF
         input(8)=1
         GO TO 7
c ..... Av_max, Av_min
      ELSE IF (cname.eq.param(9)) THEN
         READ(line,*) avmin
         input(9)=1
         GO TO 7
      ELSE IF (cname.eq.param(10)) THEN
         READ(line,*) avmax
         input(10)=1
         GO TO 7
c ..... Reddening step
      ELSE IF (cname.eq.param(11)) THEN
         READ(line,*) averr
         input(11)=1
         GO TO 7
c ..... Output file
      ELSE IF (cname.eq.param(12)) THEN
         READ(line,'(A)') spectra
         iblank=INDEX(spectra,' ')
         spectra0=spectra(1:iblank-1)//'.zspec'
         spectra1=spectra(1:iblank-1)//'.zobs_sed'
         spectra2=spectra(1:iblank-1)//'.ztemp_sed'
         filterout=spectra(1:iblank-1)//'.out_zspec'
         input(12)=1
         GO TO 7
c ..... Cosmological parameters to compute the universe age and M_abs 
      ELSE IF (cname.eq.param(13)) THEN
         READ(line,*) h0
         input(13)=1
         GO TO 7
      ELSE IF (cname.eq.param(14)) THEN
         READ(line,*) om
         input(14)=1
         GO TO 7
      ELSE IF (cname.eq.param(15)) THEN
         READ(line,*) ov
         input(15)=1
         GO TO 7
c ..... Filter for the absolute magnitude
      ELSE IF (cname.eq.param(16)) THEN
         READ(line,*) nfilt_mabs
         input(16)=1
         GO TO 7
c ..... File containing photometric catalogue
      ELSE IF (cname.eq.param(17)) THEN
         READ(line,'(A)') cat_in
         input(17)=1
         GO TO 7
c ..... Check the age of the object < universe age (y/n)
      ELSE IF (cname.eq.'AGE_CHECK') THEN
         WRITE(*,*) 'AGE_CHECK is obsolete, but it is OK for this time'
         cage=line(1:1)
         IF (cage.eq.'y'.or.cage.eq.'Y') THEN
            iage_check=1
         ELSE
            iage_check=0
         END IF
         input(18)=1
         GO TO 7
c ..... Constraint for age (it will supersede AGE_CHECK)
c       0. free
c       1. free with age <  age universe-age(T_FORM)
c       2. fixed age: age=age of universe-T(Z_FORM)
      ELSE IF (cname.eq.param(18)) THEN
         READ(line,*) iage_check
         IF (iage_check.ne.0.and.iage_check.ne.1.and.
     &        iage_check.ne.2) THEN
            WRITE(*,*) 'AGE_TYPE must be 0, 1 or 2'
            STOP
         END IF
         input(18)=1
         GO TO 7
c ..... Limits for absolute magnitude 
      ELSE IF (cname.eq.param(19)) THEN
         READ(line,*) amabs_1
         input(19)=1
         GO TO 7
      ELSE IF (cname.eq.param(20)) THEN
         READ(line,*) amabs_2
         input(20)=1
         GO TO 7
c ..... Number of the col. after the photometric cat. corresponding to zspec
      ELSE IF (cname.eq.param(21)) THEN
         READ(line,*) ncolzs
         input(21)=1
         GO TO 7
c ..... File .zobs_sed (y/n)
      ELSE IF (cname.eq.param(22)) THEN
         csed1=line(1:1)
         input(22)=1
         GO TO 7
c ..... File .ztemp_sed (y/n)
      ELSE IF (cname.eq.param(23)) THEN
         csed2=line(1:1)
         input(23)=1
         GO TO 7
c ..... Matrix/obj
      ELSE IF (cname.eq.param(24)) THEN
         cout_m=line(1:1)
         input(24)=1
         GO TO 7
c ..... Spectrum/obj
      ELSE IF (cname.eq.param(25)) THEN
         cout_s=line(1:1)
         input(25)=1
         GO TO 7
c ..... Output SED: 
c       0 -> erg/s/cm^2/A ; 1 -> microJansky ; 2 -> AB magnitudes
      ELSE IF (cname.eq.param(26)) THEN
         READ(line,*) iunit_sed
         IF (iunit_sed.ne.0.and.iunit_sed.ne.1.and.iunit_sed.ne.2) THEN
            WRITE(*,*) 'OUTPUT_TYPE must be 0, 1 or 2'
            STOP
         END IF
         IF (iunit_sed.eq.0) THEN
            cout='cgs'
         ELSE IF (iunit_sed.eq.1) THEN
            cout='muJy'
         ELSE IF (iunit_sed.eq.2) THEN
            cout='AB'
         END IF
         input(26)=1
         GO TO 7
c ..... Algorithm to compute absolute magnitudes: 
c       0. nearest filter 1. best fit normalization
      ELSE IF (cname.eq.param(27)) THEN
         READ(line,*) imabs 
         IF (imabs.ne.0.and.imabs.ne.1) THEN
            WRITE(*,*) 'MAG_ABS_ALG must be 0 or 1'
            STOP
         END IF
         input(27)=1
         GO TO 7
c ..... Transmission of absolute magnitude filter: 
c       0. energy 1. photons
      ELSE IF (cname.eq.param(28)) THEN
         READ(line,*) iband_abs
         IF (iband_abs.ne.0.and.iband_abs.ne.1) THEN
            WRITE(*,*) 'TRANS_ABS must be 0 or 1'
            STOP
         END IF
         input(28)=1
         GO TO 7
c ..... Maximum rest frame wavelength [Angstrom] to be used, according to 
c       parameter IR_FILTERS
      ELSE IF (cname.eq.param(29)) THEN
         READ(line,*) wl_ir
         input(29)=1
         GO TO 7
c ..... Possibilities using IR_FILTERS
c       0. use all available filters
c       1. use filters with lambda_eff > WL_MAX tot only when 
c          lambda_eff/(1+z) < WL_MAX
c       2. do not use filters with lambda_eff > WL_MAX
      ELSE IF (cname.eq.param(30)) THEN
         READ(line,*) ir_filters
         IF (ir_filters.ne.0.and.ir_filters.ne.1
     &        .and.ir_filters.ne.2) THEN
            WRITE(*,*) 'IR_FILTERS must be 0, 1 or 2'
            STOP
         END IF
         input(30)=1
         GO TO 7
c ..... Minimum rest frame wavelength [Angstrom] to be used, according to 
c       parameter UV_FILTERS
      ELSE IF (cname.eq.param(31)) THEN
         READ(line,*) wl_uv
         input(31)=1
         GO TO 7
c ..... Possibilities using UV_FILTERS
c       0. use all available filters
c       1. use filters with lambda_eff < WL_MIN tot only when 
c          lambda_eff/(1+z) > WL_MIN
c       2. do not use filters with lambda_eff < WL_MIN
      ELSE IF (cname.eq.param(32)) THEN
         READ(line,*) uv_filters
         IF (uv_filters.ne.0.and.uv_filters.ne.1
     &        .and.uv_filters.ne.2) THEN
            WRITE(*,*) 'UV_FILTERS must be 0, 1 or 2'
            STOP
         END IF
         input(32)=1
         GO TO 7
c ..... Formation redshift (used only if AGE_TYPE=1 or 2)
      ELSE IF (cname.eq.param(33)) THEN
         READ(line,*) z_form
         input(33)=1
         GO TO 7
c ..... Minimum age to be considered
      ELSE IF (cname.eq.param(34)) THEN
         READ(line,*) age_min
         input(34)=1
         GO TO 7
c ..... Maximum age to be considered
      ELSE IF (cname.eq.param(35)) THEN
         READ(line,*) age_max
         input(35)=1
         GO TO 7
c ..... Age rebinning (only for BC templates): 
c       0. no (221 ages) 1. yes (51 ages)
      ELSE IF (cname.eq.param(36)) THEN
         READ(line,*) irebin 
         IF (irebin.ne.0.and.irebin.ne.1) THEN
            WRITE(*,*) 'AGE_REBIN must be 0 or 1'
            STOP
         END IF
         input(36)=1
         GO TO 7
c ..... Signal-to-noise ratio limit for detection/non detection
      ELSE IF (cname.eq.param(37)) THEN
         READ(line,*) sn_lim
         input(37)=1
         GO TO 7
c ..... File containing used provided extinction law (lambda[A],k(lambda)/RV)
      ELSE IF (cname.eq.param(38)) THEN
         READ(line,'(A)') filered
         input(38)=1
         GO TO 7
      ELSE 
         WRITE(*,'(a,1x,a)') 'Unknown parameter (ignored)',cname
         GO TO 7
      END IF

 10   CONTINUE
      CLOSE(77)
      iout_m=1
      iout_s=1
      IF (cout_m.eq.'y'.or.cout_m.eq.'Y') iout_m=2
      IF (cout_s.eq.'y'.or.cout_s.eq.'Y') iout_s=2
      IF (law_rd.eq.0) THEN
         avmin=0.0D0
         avmax=0.0D0
         averr=1.0D0
      END IF
      OPEN(20,file=filered,status='old',iostat=ierr)
      IF (ierr.gt.0.and.law_rd.eq.-1) THEN
         iblank=INDEX(line,' ')
         WRITE(*,'(a)') 'User provided extinction law'
         WRITE(*,'(a)') 'File '//filered(1:iblank-1)//' not found'
         STOP
      END IF
      CLOSE(20)
      nred=NINT(ABS(avmax-avmin)/averr)+1
      IF (ABS(avmin+averr*(nred-1)-avmax).gt.1.D-6) nred=nred-1
      IF (nred.gt.mxred) THEN
         WRITE(*,'(1x,a,i3,a)') 
     .        'Number of reddening steps exceeding the maximum [max=',
     .        mxred,']'
         STOP
      END IF

      age_form=tz(h0,om,ov,z_form) ! age of universe at z_form

      OPEN(10,FILE=filedef,STATUS='old',ERR=78)
      iii=1
 2230 READ(10,'(A)',ERR=2220,END=2220) line_sed
      IF (line_sed(1:1).eq.'#'.or.line_sed.eq.' ') GO TO 2230
      last=INDEX(line_sed,' ')-1
      filesed(iii)=line_sed(1:last)
      line_sed=line_sed(last+1:)
      DO i=1,160
         IF (line_sed(i:i).ne.' ') THEN
            line_sed=line_sed(i:)
            GO TO 2221
         END IF
      END DO
      WRITE(*,*) 'Missed type of SED file ', filesed(iii)
      STOP
 2221 sed_abc(iii)=line_sed(1:2)
      iii=iii+1
      GO TO 2230
 2220 n_files=iii-1
      IF (n_files.gt.mxtyp) THEN
         WRITE(*,'(1x,a,i3,a)') 
     .        'Number of files exceeding the maximum [max=',mxtyp,']'
         STOP
      END IF
      WRITE(*,'(1x,a,1x,i4,1x,a)') 'There are ',n_files,
     &     ' synthetic spectra'
      CLOSE(10)

      OPEN(10,FILE=filt_file,STATUS='old',ERR=80)

      ifilt=0
 134  CONTINUE
      READ(10,'(A)',ERR=135,END=135) line
      IF (line(1:1).eq.'#'.or.line.eq.' ') GO TO 134
      ifilt=ifilt+1
      ncomp=1
      DO k=1,80
         IF (line(k:k).eq.'&') THEN
            line(k:k)=' '
            ncomp=ncomp+1
         ELSE IF (line(k:k).eq.'#') THEN
            GO TO 136
         END IF
      END DO
 136  CONTINUE
      IF (ncomp.eq.1) THEN 
         READ(line,*,ERR=82) nfil(ifilt),c_corr(ifilt),law_nd(ifilt),
     .        mag_nd(ifilt),err_nd(ifilt)
         kfil(1)=nfil(ifilt)
      ELSE 
         READ(line,*,ERR=82) (kfil(j),j=1,ncomp),c_corr(ifilt),
     .        law_nd(ifilt),mag_nd(ifilt),err_nd(ifilt)
      END IF
      IF (law_nd(ifilt).ne.0.and.law_nd(ifilt).ne.1.and.
     .     law_nd(ifilt).ne.2.and.law_nd(ifilt).ne.3.and.
     .     law_nd(ifilt).ne.4) THEN
         WRITE(*,'(a)') 'Non detection law must be 0, 1, 2, 3 or 4'
         WRITE(*,'(a,i3)') 'Check filter ', nfil(ifilt)
         IF (law_nd(i).eq.3.and.law_err.eq.2) THEN
            WRITE(*,'(a)') 
     .           'Cannot use simultaneously ERR_MIN_LAW=2 in main '
     .           //'parameters file and law_nd=3 in filters file'
         END IF
         STOP
      END IF
c read_filt subroutine 
      CALL read_filt(ncomp,kfil,wl(ifilt),surf(ifilt),bp1(ifilt),iband)
      GO TO 134

 135  n_filt=ifilt
      IF (n_filt.gt.mxfil) THEN
         WRITE(*,'(1x,a,i3,a)') 
     .        'Number of filters exceeding the maximum [max=',mxfil,']'
         STOP
      END IF
      WRITE(*,'(1x,a,1x,i4,1x,a)') 'There are ',n_filt,' filters'

      IF (law_err.eq.2) THEN
         DO i=1,n_filt
            err_minf(i)=err_nd(i)
         END DO
      END IF

      ifilt=ifilt+1             ! hereafter ifilt is the abs mag filter
      nfil(ifilt)=nfilt_mabs    ! filter for absolute magnitude estimates
      kfil(1)=nfil(ifilt)
      CALL read_filt(1,kfil,wl(ifilt),surf(ifilt),bp1(ifilt),iband_abs)

      CLOSE(10)

      OPEN(10,FILE=cat_in,STATUS='old',ERR=79)
      header = .FALSE.
      READ(10,'(A)',ERR=79) header_line
      IF (header_line(1:1).eq.'#'.or.header_line.eq.' ') THEN
         header= .TRUE.
         i=1
 44      READ(10,'(A)',ERR=79) header_line
         IF (header_line(1:1).eq.'#'.or.header_line.eq.' ') THEN
            i=i+1
            GO TO 44
         ELSE
            n_head=i
         END IF
      END IF
      CLOSE(10)
      OPEN(10,FILE=cat_in,STATUS='old',ERR=79)
      IF (header) THEN
         DO i=1,n_head
            READ(10,'(A)') header_line
         END DO
      END IF

      iob=0
      zmax=0.
 1340 CONTINUE
      iob=iob+1
      READ(10,*,ERR=1350,END=1350) id,(rmag(i),i=1,n_filt),
     .     (rerr_p(i),i=1,n_filt),(rcol(i),i=1,ncolzs)
      rzs=rcol(ncolzs)
      IF (rzs.gt.zmax) zmax=rzs
      GO TO 1340
 1350 nob=iob-1
      WRITE(*,'(1x,a,1x,i7,1x,a)') 'There are ',nob,
     &     ' objects in the catalogue'
      CLOSE(10)

c -----------------------------------------------------------------------
c                       Reading synthetic spectra 
c -----------------------------------------------------------------------
c ..... Vega
      WRITE (*,'(1x,a)') 'Vega SED:'
      OPEN(10,FILE=spec_vega,STATUS='old')
      READ(10,'(A)') line
      WRITE(*,'(a)') line(1:80)
c      scl=2.D-17
      scl=1.D0
      iii=1
 223  READ(10,*,ERR=222,END=222) wv,fv
      w_vega(iii)=DBLE(wv)
      f_vega(iii)=DBLE(fv)*scl
      iii=iii+1
      GO TO 223
 222  iw_vega=iii-1
      CLOSE(10)
      WRITE(*,'(1x,a,1x,i6,1x,a)') 'There are ',iw_vega,
     &     ' points in Vega'

c maximum and minimum wavelength in filters
      zmin=0.
      wlmax=0.D0
      wlmin=1.D12
      DO i=1,n_filt
         wlmax=MAX(wlmax,r(np(i),i,1))
         wlmin=MIN(wlmin,r(1,i,1))
      END DO
c maximum wavelength at the maximum redshift
      wlmax=wlmax*(1.D0+zmax)
c minimum wavelength at the minimum redshift
      wlmin=wlmin*(1.D0+zmin)

c ..... SED
      WRITE(*,*) 'Reading synthetic spectra :'
      i_bc=0
      i_as=0
      i_pg=0
      DO i=1,n_files
         ntyp_bc(i)=-1
         ntyp_as(i)=-1
         ntyp_pg(i)=-1
      END DO 
   
      DO i=1,n_files
         last=INDEX(filesed(i),' ')
         WRITE(*,2005) i,filesed(i)(1:last)
c GISSEL's templates
         IF (sed_abc(i).eq.'BC') THEN
            OPEN(1,FILE=filesed(i),STATUS='old',
     .           FORM='unformatted',ERR=25)
            i_bc=i_bc+1
            IF (i_bc.gt.mxtyp_bc) THEN
               WRITE(*,'(a,i3)') 
     .              ' Number of GISSEL templates exceeding the maximum.' 
     .              //' Max = ',mxtyp_bc
               WRITE(*,'(a)') 
     .              ' Reduce GISSEL SEDs or modify mxtyp_bc in '
     .              //'dimensionz.dec and recompile'
               STOP
            END IF
            ntyp_bc(i)=i_bc
c ks = n of temporal intervals, t_bc and t_bc0 in [yr]
            READ (1) ks(i),(t_bc0(ii),ii=1,ks(i)),ml,mu,iseg,
     &	   (xx(j),lm(j),um(j),baux(j),cn(j),cc(j),j=1,iseg),
     &	   totm,totn,avs,jo,tau0,string_id
c replace value of tau if SFH is not exponentially declining
c SFHs following jo parameter: 
c jo=0: SSP; jo=1: exp. declining with gas recycling; 
c jo=2: burst of finite length; jo=3: constant; jo=4: exp. declining; 
c jo=5: burst added; jo=6: delayed; jo=7: input SFH; 
c jo=8: linearly decreasing
         IF (jo.eq.0) THEN      ! SSP 
            tau(i)=0.D0
         ELSE IF (jo.eq.3) THEN
            tau(i)=-1.D0        ! constant SFR
         ELSE IF (jo.eq.4.or.jo.eq.1) THEN
            tau(i)=tau0/1.D9         ! TAU models
         ELSE
            tau(i)=-10.D0       ! other SFHs
         END IF
         ic1=INDEX(string_id,'Z=')
         string_id=string_id(ic1+2:)
         read(string_id,*) zsun(i) 
c iw = number of wavelengths, w_bc wavelength in [A]
            READ (1) iw(i),(w_bc(ii,i_bc),ii=1,iw(i)) 
            IF (iw(i).gt.mxwl_bc) THEN
               WRITE(*,'(a,i6)') 'Number of wavelengths in GISSEL '
     .              //'templates exceeding the maximum. Max = ',mxwl_bc
               WRITE(*,'(a)') 
     .              ' Reduce GISSEL wavelenghts or modify mxwl_bc in '
     .              //'dimensionspec.dec and recompile'
               STOP
            END IF
            k=0
            DO j=1,ks(i)
               READ (1) iw(i),(ff(ii),ii=1,iw(i))
               IF ((irebin.eq.1.and.(t_bc0(j)/1.E9.le.0.01.and.
     &              (FLOAT(j)/10.-FLOAT(j/10)).lt.1.E-6)
     &              .or.(t_bc0(j)/1.E9.gt.0.01.and.
     &              t_bc0(j)/1.E9.le.3..and.
     &              (FLOAT(j-49)/3.-FLOAT((j-49)/3)).lt.1.E-6)
     &              .or.(t_bc0(j)/1.E9.gt.3..and.
     &              t_bc0(j)/1.E9.le.20..and.
     &              (FLOAT(j-15)/4.-FLOAT((j-15)/4)).lt.1.E-6)).or.
     &              irebin.eq.0) THEN
                  k=k+1
                  t_bc(k)=DBLE(t_bc0(j))
                  DO ii=1,iw(i)
                     IF (w_bc(ii,i_bc).lt.912.) THEN
                        f_bc(ii,k,i_bc)=0.D0
                     ELSE
                        f_bc(ii,k,i_bc)=DBLE(ff(ii))
                     END IF
                  END DO
               END IF
            END DO
            ks(i)=k
            CLOSE(1)

c ASCII SEDs
         ELSE IF (sed_abc(i).eq.'AS'.or.sed_abc(i).eq.'A ') THEN
            OPEN(1,FILE=filesed(i),STATUS='old',
     .           FORM='formatted',ERR=25)
            i_as=i_as+1
            IF (i_as.gt.mxtyp_as) THEN
               WRITE(*,'(a,i3)') 
     .              ' Number of ASCII templates exceeding the maximum.' 
     .              //' Max = ',mxtyp_as
               WRITE(*,'(a)') 
     .              ' Reduce ASCII SEDs or modify mxtyp_as in '
     .              //'dimensionspec.dec and recompile'
               STOP
            END IF
            ntyp_as(i)=i_as
            ks(i)=1
            ii=1

 33         READ(1,'(A)',ERR=34,END=34) line_sed
            IF (line_sed(1:1).eq.'#'.or.line_sed.eq.' ') GO TO 33

            READ(line_sed,*) w_as(ii,i_as),fff
c            f_as(ii,1,i_as)=DBLE(fff)
            f_as(ii,1,i_as)=fff
            ii=ii+1
            IF (ii.gt.mxwl_as) THEN
               WRITE(*,'(a,i6)') 'Number of wavelengths in ASCII '
     .              //'templates exceeding the maximum. Max = ',mxwl_as
               WRITE(*,'(a)') 
     .              ' Reduce ASCII wavelenghts or modify mxwl_as in '
     .              //'dimensionspec.dec and recompile'
               STOP
            END IF
            GO TO 33
 34         iw(i)=ii-1
            CLOSE(1)

c PEGASE templates
         ELSE IF (sed_abc(i).eq.'PG') THEN
            OPEN(1,FILE=filesed(i),STATUS='old',
     .           FORM='formatted',ERR=25)
            i_pg=i_pg+1
            IF (i_pg.gt.mxtyp_pg) THEN
               WRITE(*,'(a,i3)') 
     .              ' Number of PEGASE templates exceeding the maximum.' 
     .              //' Max = ',mxtyp_pg
               WRITE(*,'(a)') 
     .              ' Reduce PEGASE SEDs or modify mxtyp_pg in '
     .              //'dimensionspec.dec and recompile'
               STOP
            END IF
            ntyp_pg(i)=i_pg
 35         READ (1,'(a)',ERR=36,END=36) line_pg 
            IF (line_pg(1:5).eq.'*****') GO TO 37
            GO TO 35
 37         READ (1,*,ERR=36,END=36) ks(i),iw(i),nemline
            IF (ks(i).gt.mxage_pg) THEN
               WRITE(*,'(a,i4)') 
     .              ' Number of ages in PEGASE templates exceeding '
     .              //'the maximum. Max = ', mxage_pg
               WRITE(*,'(a)') 
     .              ' Change the SED or modify mxage_pg in '
     .              //'dimensionspec.dec and recompile'
               STOP
            END IF
            IF (iw(i).gt.mxwl_pg) THEN
               WRITE(*,'(a,i6)') 'Number of wavelengths in PEGASE '
     .              //'templates exceeding the maximum. Max = ',mxwl_pg
               WRITE(*,'(a)') 
     .              ' Reduce PEGASE wavelenghts or modify mxwl_pg in '
     .              //'dimensionspec.dec and recompile'
               STOP
            END IF
            READ(1,*) (w_pg(j,i_pg),j=1,iw(i))
            READ(1,*) (aa8,j=1,nemline)
            DO j=1,ks(i)
c 1st line: 1.age[Myr](int), 2.Mgal[Msun], 3.Mstar[Msun], 4.MWD[Msun],
c           5.MBHNS[Msun], 6.Msub[Msun], 7.Mgas[Msun], 8.Zgas, 
c           9.<Zstars>_mass, 10.<Zstars>_Lbol
               READ(1,*) it_pg,amgal,amstar,(aa8,k=1,7)
c 2nd line: 1.L_bol[erg/s], 2. tauV, 3.Ldust/Lbol, 4.SFR[Msun/Myr],
c           5. nLymcont[s-1], 6.nSNII[Myr-1], 7.nSNIa[Myr-1], 
c           8.<age>_mass[Myr], 9.<age>_Lbol[Myr]
               READ(1,*) aa8,aa8,aa8,sfr,aa8,aa8,aa8,aa8,aa8
               READ(1,*) (ff(k),k=1,iw(i))
               t_pg(j)=DBLE(it_pg) ! it_pg is age[Myr] (integer)
               DO k=1,iw(i)
                  f_pg(k,j,i_pg)=DBLE(ff(k))
                  IF (j.eq.77.and.ntyp_pg(i).eq.1) 
     &             write(94,'(3(1x,i3),2(1x,es12.5))')
     &                 i_pg,it_pg,ntyp_pg(i),w_pg(k,i_pg),
     &                    f_pg(k,j,i_pg)/lsun
               END DO
               READ(1,*) (aa,k=1,nemline)
            END DO
c normalise in solar units
            DO k=1,ks(i)
               DO ii=1,iw(i)
                  f_pg(ii,k,i_pg)=f_pg(ii,k,i_pg)/lsun
               END DO
            END DO
c Pegase SEDs already = 0 at lambda < 912.A
            DO k=1,ks(i)
               DO ii=1,iw(i)
                  IF (w_pg(ii,i_pg).lt.912.) f_pg(ii,k,i_pg)=0.
               END DO
            END DO
 36         CLOSE(1)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
         ELSE
            WRITE(*,*) 'Don''t know how to read :'
            WRITE(*,'(A)') filesed(i)(1:INDEX(filesed(i),' '))
            STOP
         END IF
      END DO
 2005 FORMAT(' Type ', I3, ' : ', A)
      WRITE(*,'(1x,a,i4,a)') 'Read ',i_bc,' GISSEL SEDs'
      WRITE(*,'(1x,a,i4,a)') '     ',i_as,' ASCII  SEDs'
      WRITE(*,'(1x,a,i4,a)') '     ',i_pg,' PEGASE SEDs'
c -----------------------------------------------------------------------
c                    Filters and .out_phot file
c -----------------------------------------------------------------------

      OPEN(18,file=filterout,status='unknown')
      CALL fdate(date)
c      CALL fdate_(date)   ! IBM
      WRITE(18,'(a)') '# Date: '//date
      WRITE(18,'(a)') '# Photometric catalogue : '
      last=INDEX(cat_in,' ')-1
      WRITE(18,'(2x,a)')  cat_in(1:last)
      WRITE(18,'(a)') 
     .     '# Templates SEDs:'
      DO itype=1,n_files
         IF (sed_abc(itype).eq.'BC') THEN
            WRITE(18,'(2x,i4,2x,a,2x,3a,2x,f9.4,f9.5)') itype,
     .           filesed(itype)(1:INDEX(filesed(itype),' ')),
     .           sed_abc(itype),tau(itype),zsun(itype)
         ELSE
            WRITE(18,'(2x,i4,2x,a,2x,3a)') itype,
     .           filesed(itype)(1:INDEX(filesed(itype),' ')),
     .           sed_abc(itype)
         END IF
      END DO
      WRITE(18,'(a)') '# Characteristics of SED fitting filters'
      WRITE(18,'(a)') '#  n      wl_eff    surface  bandpass1  conv_AB'
     .     //' m_corr mlim(AB)   nd trans dm_min dm_law mtyp'
      WRITE(18,'(a)') '#  trans: 0 energy, 1 photons'
      zz=0.D0

c ..... Conversion from AB to standard magnitudes: conv_mag = AB - Vega
c computing AB magnitude of Vega
      DO k=1,n_filt+1
         vega(k)=filter(k,w_vega,f_vega,iw_vega,zz)
         DO i=1,np(k)
            xlam(i)=r(i,k,1) ! lambda
            rr2(i)=r(i,k,2)/(xlam(i)*xlam(i)) ! R/lambda^2
         END DO
         f2(k)=c*trapz1(xlam,rr2,np(k))
         conv_mag(k)=-2.5D0*DLOG10(vega(k)/f2(k))-48.60D0
      END DO
      DO k=1,n_filt
c ..... Flux corresponding to the limiting magnitude
         IF (magtyp.eq.0) THEN
            mag_nd1(k)=mag_nd(k) ! limiting mag written in filters file
            fl_nd(k)=vega(k)*
     .           10**(-0.4D0*(mag_nd(k)+c_corr(k)))/surf(k) 
c            mag_nd(k)=-2.5D0*DLOG10(fl_nd(k)*surf(k)/vega(k)) !Vega lim mag
            mag_nd(k)=mag_nd1(k)+c_corr(k)+conv_mag(k) ! AB limiting mag
         ELSE IF (magtyp.eq.1) THEN
            mag_nd1(k)=mag_nd(k)
            fl_nd(k)=10**(-0.4D0*(mag_nd1(k)+c_corr(k)+48.6D0))
            mag_nd(k)=mag_nd1(k)+c_corr(k) ! AB limiting mag
         ELSE IF (magtyp.eq.2) THEN 
            fl_nd(k)=(mag_nd(k)+c_corr(k))*1.D-29 
            mag_nd(k)=-2.5D0*DLOG10(fl_nd(k))-48.6 ! AB limiting mag
         END IF
         IF (law_err.eq.2) THEN
            WRITE(18,1998) nfil(k),wl(k),surf(k),bp1(k),conv_mag(k),
     .           c_corr(k),mag_nd(k),law_nd(k),iband,err_minf(k),
     .           law_err,cmag
         ELSE
            WRITE(18,1998) nfil(k),wl(k),surf(k),bp1(k),conv_mag(k),
     .           c_corr(k),mag_nd(k),law_nd(k),iband,err_min,law_err,
     .           cmag
         END IF
      END DO
 1998 FORMAT(I4,1X,F11.3,1X,2(F10.3,1X),1X,F7.3,1X,F6.3,1X,F7.3,
     .     4X,I2,4X,I2,2X,F5.3,3X,I2,3X,A5)
      IF (law_rd.eq.-1) THEN
         WRITE(18,'(a)') '#    AVmin     AVmax    AVstep  red_law file'
         WRITE(18,'(4X,3(F5.2,5X),I2,2x,A)') avmin,avmax,averr,law_rd,
     .        filered(1:INDEX(filered,' '))
      ELSE
         WRITE(18,'(a)') '#    AVmin     AVmax    AVstep  red_law'
         WRITE(18,'(4X,3(F5.2,5X),I2)') avmin,avmax,averr,law_rd
      END IF
      WRITE(18,'(a)') 
     .     '#    Lyman_forest [factor(s) multiplying <tau_eff>]'
      IF (nlyf.eq.1) WRITE(18,'(4X,F6.3)') r_lyf
      IF (nlyf.eq.3) WRITE(18,'(4X,3(F6.3,2X))') 
     .     1.D0/r_lyf,r_lyf/r_lyf,r_lyf
      WRITE(18,'(a)') '# nMabs   wl_eff    surface  bandpass1'
     .     //'   conv_AB     Mmin   Mmax algorithm '
     .     //'trans out_typ'
      WRITE(18,1999) nfilt_mabs,wl(ifilt),surf(ifilt),bp1(ifilt),
     .     conv_mag(ifilt),amabs_1,amabs_2,imabs,iband_abs,cout
 1999 FORMAT(I4,1X,F11.3,1X,2(F10.3,1X),1X,F7.3,2(2X,F8.2),2(4X,I2),
     .     2X,A5)
      WRITE(18,'(a)') '#    Age_type age_min[Gyr] age_max[Gyr]  z_form'
      WRITE(18,'(6X,I3,6X,F7.3,6X,F7.3,6X,F6.2)') 
     .     iage_check,age_min,age_max,z_form
      WRITE(18,'(a)') '#    lambda_max[A]   IR [0.use all; 1.use if '
     .     //'lambda/(1+z)<lambda_max; 2.use if lambda<lambda_max]'
      WRITE(18,'(6X,F10.2,4x,I2)') wl_ir,ir_filters
      WRITE(18,'(a)') '#    lambda_min[A]   UV [0.use all; 1.use if '
     .     //'lambda/(1+z)>lambda_min; 2.use if lambda>lambda_min]'
      WRITE(18,'(6X,F10.2,4x,I2)') wl_uv,uv_filters
      WRITE(18,'(a)') '#    Omega_0   Omega_v  H_0   minimum S/N'
      WRITE(18,'(4X,3(F5.2,5X),2X,F5.2)') om,ov,h0,sn_lim
c -----------------------------------------------------------------------
c                          Loop over the objects 
c -----------------------------------------------------------------------
      WRITE(*,'(1x,a)') 'I''m working ...'
      OPEN(13,FILE=spectra0,STATUS='unknown',ERR=81)
      DO i=1,n_filt
         IF (i.le.9) THEN
            WRITE(ch(i),'(i1)') i
         ELSE IF (i.ge.10.and.i.le.99) THEN
            WRITE(ch(i),'(i2)') i
         ELSE 
            WRITE(ch(i),'(i3)') i
         END IF
         IF (magtyp.eq.0.or.magtyp.eq.1) THEN
            mags(i)='mag'//ch(i)
            dmags(i)='dmag'//ch(i)
         ELSE IF (magtyp.eq.2) THEN
            mags(i)='fl'//ch(i)
            dmags(i)='dfl'//ch(i)
         END IF         
      END DO
      WRITE(fmt0,2010) n_filt
      WRITE(fmt00,2010) 2*n_filt
      IF (magtyp.eq.0.or.magtyp.eq.1) THEN
         WRITE(13,fmt0) 
     .        '#         ID   redshift    chi2 P_chi2 iSED iage'
     .        //' age[Gyr]  A_V       norm     ',
     .        (mags(i)(1:9),i=1,n_filt),' Mabs Nfilt   D_L[Mpc]'
      ELSE IF (magtyp.eq.2) THEN
         WRITE(13,fmt0) 
     .        '#         ID   redshift    chi2 P_chi2 iSED iage'
     .        //' age[Gyr]  A_V       norm     ',
     .        (mags(i)(1:12),i=1,n_filt),' Mabs Nfilt   D_L[Mpc]'
      END IF    
      IF (csed1.eq.'y'.or.csed1.eq.'Y') THEN
         OPEN(14,FILE=spectra1,STATUS='unknown',ERR=81)
         IF (iunit_sed.eq.0.or.iunit_sed.eq.1) THEN
            DO i=1,n_filt
               IF (i.le.9) THEN
                  WRITE(ch(i),'(i1)') i
               ELSE IF (i.ge.10.and.i.le.99) THEN
                  WRITE(ch(i),'(i2)') i
               ELSE 
                  WRITE(ch(i),'(i3)') i
               END IF
               mags(i)='fl'//ch(i)
               dmags(i)='dfl'//ch(i)
            END DO
         END IF
         IF (iunit_sed.eq.0) THEN
            WRITE(14,fmt00) '#          ID          ',
     .           (mags(i)(1:13),i=1,n_filt),
     .           (dmags(i)(1:12),i=1,n_filt),' [erg/cm^2/s]'
         ELSE IF (iunit_sed.eq.1) THEN
            WRITE(14,fmt00)  '#          ID          ',
     .           (mags(i)(1:13),i=1,n_filt),
     .           (dmags(i)(1:12),i=1,n_filt),' [muJy]'
         ELSE IF (iunit_sed.eq.2) THEN
            WRITE(14,fmt00)  '#          ID     ',
     .           (mags(i)(1:9),i=1,n_filt),
     .           (dmags(i)(1:9),i=1,n_filt),' [AB mag]'
         END IF
      END IF
      IF (csed2.eq.'y'.or.csed2.eq.'Y') THEN
         OPEN(15,FILE=spectra2,STATUS='unknown',ERR=81)
         IF (iunit_sed.eq.0.or.iunit_sed.eq.1) THEN
            DO i=1,n_filt
               IF (i.le.9) THEN
                  WRITE(ch(i),'(i1)') i
               ELSE IF (i.ge.10.and.i.le.99) THEN
                  WRITE(ch(i),'(i2)') i
               ELSE 
                  WRITE(ch(i),'(i3)') i
               END IF
               mags(i)='fl'//ch(i)
            END DO
         END IF
         IF (iunit_sed.eq.0) THEN
            WRITE(15,fmt0) '#          ID          ',
     .           (mags(i)(1:13),i=1,n_filt),' [erg/cm^2/s]'
         ELSE IF (iunit_sed.eq.1) THEN
            WRITE(15,fmt0)  '#          ID          ',
     .           (mags(i)(1:13),i=1,n_filt),' [muJy]'
         ELSE IF (iunit_sed.eq.2) THEN
            WRITE(15,fmt0)  '#          ID     ',
     .           (mags(i)(1:9),i=1,n_filt),' [AB mag]'
         END IF
      END IF

      IF (magtyp.eq.0.or.magtyp.eq.1) THEN 
         WRITE(fmt,2001) n_filt
      ELSE IF (magtyp.eq.2) THEN
         WRITE(fmt,2002) n_filt
      END IF
      WRITE(fmt1,2006) n_filt,n_filt
      WRITE(fmt2,2007) n_filt,n_filt
      WRITE(fmt3,2008) n_filt
      WRITE(fmt4,2009) n_filt
 2001 FORMAT ('(1X,I12,2X,F9.5,1X,F8.3,1X,F6.2,1X,I3,1X,I3,1X,
     &     F8.4,1X,F5.2,1X,ES10.4,',i2,'(1X,F8.4),1X,F9.4,1X,I3,1X,
     &     F11.3)')
 2002 FORMAT ('(1X,I12,2X,F9.5,1X,F8.3,1X,F6.2,1X,I3,1X,I3,1X,
     &     F8.4,1X,F5.2,1X,ES10.4,',i2,'(1X,ES11.4),1X,F9.4,1X,I3,1X,
     &     F11.3)')
 2006 FORMAT('(1X,I12,',i2,'(1X,F8.4),',i2,'(1x,F8.4))')
 2007 FORMAT('(1X,I12,',i2,'(1X,ES12.5),',i2,'(1x,ES12.5))')
 2008 FORMAT('(1X,I12,',i2,'(1X,F8.4))')
 2009 FORMAT('(1X,I12,',i2,'(1X,ES12.5))')
 2010 FORMAT('(A,',I3,'(A),A)')

      OPEN(16,FILE=cat_in,STATUS='old',ERR=79)
      IF (header) THEN
         DO i=1,n_head
            READ(16,'(A)') header_line
         END DO
      END IF

      iob=0
13400 CONTINUE
      iob=iob+1
      READ(16,*,ERR=13500,END=13500) id,(rmag(i),i=1,n_filt),
     .     (rerr_p(i),i=1,n_filt),(rcol(i),i=1,ncolzs)
      rzs=rcol(ncolzs)
      zs=DBLE(rzs)
      IF (zs.ge.0.D0.and.zs.lt.9.9D0) THEN
         ageuni=tz(h0,om,ov,zs)
         IF (zs.lt.1.D-4) THEN
            dl=0.D0
         ELSE
            dl=d_l(h0,om,ov,zs) !luminosity distance in Mpc
         END IF
         IF (dl.gt.0.D0) dmod=5.D0*DLOG10(dl)+25.D0
c minimum dmod: corresponding to d(~M31)=1000 Kpc
         IF (dmod.lt.25.D0) dmod=25.D0
         DO iii=1,n_filt
            IF (((magtyp.eq.0.or.magtyp.eq.1).and.
     .           (rmag(iii).lt.90.0.and.rmag(iii).gt.1.)).or.
     .           (magtyp.eq.2.and.rmag(iii).ge.0.)) GO TO 13401
         END DO
         WRITE(*,*) 'WARNING: non detected object id ', id
      END IF
      IF (zs.ge.9.9D0) dl=d_l(h0,om,ov,zs)
      r_chi2=9999.999D0
      qq=0.D0
      i_typ=0
      i_age=0
      age=99.D0
      av=0.D0
      bb=0.D0
      amagbbf=99.D0
      iw_ob=0
      DO k=1,n_filt
         amagfit(k)=99.D0
      END DO
      WRITE(13,fmt) id,zs,r_chi2,qq,i_typ,i_age,age,av,bb,
     .     (amagfit(k),k=1,n_filt),amagbbf,iw_ob,dl
c ..... File .zobs_sed
      IF (csed1.eq.'y'.or.csed1.eq.'Y') THEN
         DO i=1,n_filt
            fl1(i)=-1.D0
            err_fl(i)=0.D0
            amag(i)=99.D0
            err_p(i)=99.D0
         END DO
         IF (iunit_sed.eq.0.or.iunit_sed.eq.1) THEN
            WRITE(14,fmt2) id,(fl1(i),i=1,n_filt),
     .           (err_fl(i),i=1,n_filt)
         ELSE IF (iunit_sed.eq.2) THEN
            WRITE(14,fmt1) id,(amag(i),i=1,n_filt),
     .           (err_p(i),i=1,n_filt)
         END IF
      END IF
c ..... File .ztemp_sed
      IF (csed2.eq.'y'.or.csed2.eq.'Y') THEN 
         DO k=1,n_filt
            mag_temp(k)=99.D0
            fl_temp(k)=-1.D0
         END DO
         IF (iunit_sed.eq.0.or.iunit_sed.eq.1) THEN
            WRITE(15,fmt4) id,(fl_temp(k),k=1,n_filt)
         ELSE IF (iunit_sed.eq.2) THEN
            WRITE(15,fmt3) id,(mag_temp(k),k=1,n_filt)
         END IF
      END IF
      GO TO 13400
         
c ..... Computing the fluxes and errors in the filters
13401 iw_ob=0
      DO iii=1,n_filt
         amag(iii)=DBLE(rmag(iii))
         err_p(iii)=DBLE(rerr_p(iii))
         i_lyman(iii)=0
         sn=-1.D0
c ..... check the S/N in each filter if mag is detected
         IF (((magtyp.eq.0.or.magtyp.eq.1).and.amag(iii).gt.0.D0
     &        .and.amag(iii).lt.90.D0.and.err_p(iii).lt.90.D0.and.
     &        err_p(iii).gt.0.D0).or.(magtyp.eq.2.and.
     &        amag(iii).gt.0.D0.and.err_p(iii).gt.0.D0)) THEN 
            IF (magtyp.eq.0.or.magtyp.eq.1) THEN
               IF (err_p(iii).lt.err_min) THEN ! avoid to divide by zero
                  sn=(2.5D0/DLOG(10.D0))/err_min
               ELSE
                  sn=(2.5D0/DLOG(10.D0))/err_p(iii)
               END IF
            ELSE
               IF (err_p(iii).lt.err_min) THEN ! avoid to divide by zero
                  sn=amag(iii)/err_min
               ELSE
                  sn=amag(iii)/err_p(iii)
               END IF
            END IF
            IF (sn.lt.sn_lim) THEN ! don't use the filter
               amag(iii)=-99.D0
               err_p(iii)=99.D0
            END IF
         END IF
c photometric errors
         IF (law_err.eq.0) THEN
            IF (err_p(iii).lt.err_min) err_p(iii)=err_min
         ELSE IF (law_err.eq.1) THEN
c added in quadrature
            err_p(iii)=DSQRT(err_min**2+err_p(iii)**2)
         ELSE IF (law_err.eq.2) THEN
c added in quadrature with values given filter by filter
            err_p(iii)=DSQRT(err_minf(iii)**2+err_p(iii)**2)
            IF (err_p(iii).lt.err_min) err_p(iii)=err_min
         END IF

c ..... check if the filter is above WL_MAX
c       0. use all available filters
c       1. use filters with lambda_eff > wl_max tot only when 
c          lambda_eff/(1+z) < wl_max
c       2. do not use filters with lambda_eff > wl_max
         IF ((ir_filters.eq.1.and.wl(iii)/(1.D0+zs).gt.wl_ir).or.
     &        (ir_filters.eq.2.and.wl(iii).gt.wl_ir)) THEN 
c do not use it: put the magnitude as not observed and decrease the 
c degrees of freedom
            amag(iii)=-1.D0
            err_p(iii)=-1.D0
         END IF
c ..... check if the filter is below WL_MIN
c       0. use all available filters
c       1. use filters with lambda_eff < wl_min tot only when 
c          lambda_eff/(1+z) > wl_min
c       2. do not use filters with lambda_eff < wl_min
         IF ((uv_filters.eq.1.and.wl(iii)/(1.D0+zs).lt.wl_uv).or.
     &        (uv_filters.eq.2.and.wl(iii).lt.wl_uv)) THEN 
c do not use it: put the magnitude as not observed and decrease the 
c degrees of freedom
            amag(iii)=-1.D0
            err_p(iii)=-1.D0
         END IF

c ..... Detected object in filter iii - Vega or AB magnitudes
         IF ((magtyp.eq.0.or.magtyp.eq.1).and.
     .        (amag(iii).gt.1.D0.and.amag(iii).lt.90.D0).and.
     .        err_p(iii).lt.90.D0) THEN
            iw_ob=iw_ob+1
c Vega mag -> erg/cm2/s/A
            IF (magtyp.eq.0) fl(iii)=vega(iii)*
     .           10**(-0.4D0*(amag(iii)+c_corr(iii)))/surf(iii) 
c AB mag -> erg/cm2/s/Hz
            IF (magtyp.eq.1) fl(iii)=10**(-0.4D0*(amag(iii)+
     &           c_corr(iii)+48.6D0))
            err_fl(iii)=0.4D0*DLOG(10.D0)*fl(iii)*err_p(iii)
c amag1: reference Vega or AB magnitude for absolute magnitudes
            amag1(iii)=amag(iii)+c_corr(iii)
c ..... Detected object in filter iii - flux in microJansky
        ELSE IF (magtyp.eq.2.and.amag(iii).ge.0.D0
     .           .and.err_p(iii).ge.0.D0) THEN ! input in muJy
            iw_ob=iw_ob+1
            f_jy(iii)=amag(iii)
            err_jy(iii)=err_p(iii)
c muJy -> erg/cm2/s/Hz
            fl(iii)=(amag(iii)+c_corr(iii))*1.D-29
            err_fl(iii)=err_p(iii)*1.D-29
c amag: flux[muJy] -> AB magnitude
            amag1(iii)=-2.5*DLOG10(fl(iii))-48.60

c ..... Detected object in filter iii as upper limit - Vega or AB magnitudes
c       use non detection law 2
         ELSE IF ((magtyp.eq.0.or.magtyp.eq.1).and.
     .           (amag(iii).gt.1.D0.and.amag(iii).lt.90.D0).and.
     .           err_p(iii).gt.90.D0) THEN
            iw_ob=iw_ob+1
            IF (magtyp.eq.0) fl(iii)=vega(iii)*
     .           10**(-0.4D0*(amag(iii)+c_corr(iii)))/surf(iii) 
            IF (magtyp.eq.1) fl(iii)=10**(-0.4D0*(amag(iii)+
     &           c_corr(iii)+48.6D0))
            fl(iii)=0.5D0*fl(iii)
            err_fl(iii)=fl(iii)
            i_lyman(iii)=1
c amag1: reference Vega or AB magnitude for absolute magnitudes
               amag1(iii)=amag(iii)+c_corr(iii)
c ..... Detected object in filter iii as upper limit - flux in microJansky
c       use non detection law 2
         ELSE IF (magtyp.eq.2.and.amag(iii).ge.0..and.
     .           err_p(iii).lt.0.) THEN 
            iw_ob=iw_ob+1
            fl(iii)=(amag(iii)+c_corr(iii))*1.D-29
            err_fl(iii)=err_p(iii)*1.D-29
            amag1(iii)=-2.5*DLOG10(fl(iii))-48.60
            fl(iii)=0.5D0*fl(iii)
            err_fl(iii)=fl(iii)
            f_jy(iii)=0.5D0*amag(iii)
            err_jy(iii)=f_jy(iii)
            i_lyman(iii)=1

c ..... Non observed object in filter iii - Vega or AB magnitudes
         ELSE IF ((magtyp.eq.0.or.magtyp.eq.1).and.
     .           amag(iii).le.1.D0) THEN ! out of file objects
            fl(iii)=-1.0D0
            err_fl(iii)=0.0D0
            amag1(iii)=-1.0D0

c ..... Non observed object in filter iii - flux in microJansky
         ELSE IF (magtyp.eq.2.and.ABS(amag(iii)+1.D0).lt.1.D-6) THEN 
            fl(iii)=-1.0D0
            err_fl(iii)=0.0D0
            amag1(iii)=-1.0D0
            f_jy(iii)=-1.0D0
            err_jy(iii)=0.0D0

c ..... Non detected object in filter iii - Vega or AB magnitudes or flux 
c       use selected non detection law and input limiting magnitude or flux
         ELSE IF (((magtyp.eq.0.or.magtyp.eq.1).and.
     .           amag(iii).ge.90.D0).or.
     .           (magtyp.eq.2.and.amag(iii).lt.-90.D0)) THEN 
c ..... Non detection laws: 
c       0. the filter will not be used
c       1. the flux in the filter is set = 0 with 
c          photometric error = limiting flux 
c       2. the flux in the filter and the photometric error are set =
c          0.5*limiting flux
c       3. the flux in the filter is = limiting flux and the photometric
c          error is computed from column 5 of the parameter file
c       4. the filter will not be used in chi2, but solutions with 
c          flux > limiting flux will be rejected
            IF (law_nd(iii).eq.0) THEN
               fl(iii)=-1.0D0
               err_fl(iii)=0.D0
               amag1(iii)=99.D0
               f_jy(iii)=-1.0D0
               err_jy(iii)=0.D0
            ELSE IF (law_nd(iii).eq.1) THEN
               fl(iii)=0.D0
               err_fl(iii)=fl_nd(iii)
               amag1(iii)=99.D0
               i_lyman(iii)=2
               iw_ob=iw_ob+1
            ELSE IF (law_nd(iii).eq.2) THEN
               fl(iii)=0.5D0*fl_nd(iii)
               err_fl(iii)=fl(iii)
               amag1(iii)=99.D0
               i_lyman(iii)=1
               iw_ob=iw_ob+1
            ELSE IF (law_nd(iii).eq.3) THEN
               fl(iii)=fl_nd(iii)
               err_p(iii)=err_nd(iii)
               amag1(iii)=99.D0
               IF (magtyp.eq.0.or.magtyp.eq.1) THEN
                  err_fl(iii)=0.4D0*DLOG(10.D0)*fl(iii)*err_p(iii)
               ELSE
                  err_fl(iii)=err_p(iii)*1.D-29
               END IF
               i_lyman(iii)=1
               iw_ob=iw_ob+1
            ELSE IF (law_nd(iii).eq.4) THEN
               fl(iii)=-1.D0
               err_fl(iii)=0.D0
               amag1(iii)=99.D0
               i_lyman(iii)=2
            END IF
         END IF
      END DO

      ndeg=iw_ob-1  ! degrees of freedom
      deg=DBLE(REAL(ndeg))

      IF (ndeg.lt.1) THEN
         WRITE(*,'(a,i12)') 
     &        'WARNING: impossible to compute object ID = ', id
c ..... Write outputs in any case 
         r_chi2=9999.999D0
         qq=0.D0
         i_typ=0
         i_age=0
         age=99.D0
         av=0.D0
         bb=0.D0
         amagbbf=99.D0
         DO k=1,n_filt
            amagfit(k)=99.
         END DO
         WRITE(13,fmt) id,zs,r_chi2,qq,i_typ,i_age,age,av,bb,
     .           (amagfit(k),k=1,n_filt),amagbbf,iw_ob,dl
c ..... File .zobs_sed
         IF (csed1.eq.'y'.or.csed1.eq.'Y') THEN
            DO i=1,n_filt
               fl1(i)=-1.D0
               err_fl(i)=0.D0
               amag(i)=99.D0
               err_p(i)=99.D0
            END DO
            IF (iunit_sed.eq.0.or.iunit_sed.eq.1) THEN
               WRITE(14,fmt2) id,(fl1(i),i=1,n_filt),
     .              (err_fl(i),i=1,n_filt)
            ELSE IF (iunit_sed.eq.2) THEN
               WRITE(14,fmt1) id,(amag(i),i=1,n_filt),
     .              (err_p(i),i=1,n_filt)
            END IF
         END IF
c ..... File .ztemp_sed
         IF (csed2.eq.'y'.or.csed2.eq.'Y') THEN 
            DO k=1,n_filt
               mag_temp(k)=99.D0
               fl_temp(k)=-1.D0
            END DO
            IF (iunit_sed.eq.0.or.iunit_sed.eq.1) THEN
               WRITE(15,fmt4) id,(fl_temp(k),k=1,n_filt)
            ELSE IF (iunit_sed.eq.2) THEN
               WRITE(15,fmt3) id,(mag_temp(k),k=1,n_filt)
            END IF
         END IF
         GO TO 13400
      END IF

c -----------------------------------------------------------------------
c                         Computation 
c -----------------------------------------------------------------------
c ..... Open file for the matrix of results (if case)

      matrix=0
      root=' '
      WRITE(root,'(i12)') id
      DO k=1,12
         IF (root(k:k).ne.' ') THEN
            root=root(k:)
            GO TO 22
         END IF
      END DO
 22   iroot=INDEX(root,' ')-1
      IF (iout_m.gt.1) THEN
         amatrix=root(1:iroot)//'.zm'
         matrix=1
         OPEN(11,FILE=amatrix,STATUS='unknown')
         WRITE(11,'(a)') 
     .     '#      ID redshift iSED iage age[Gyr] A_V     chi2'
     .        //'         Pchi2      norm      Mabs'
      END IF

      DO k=1,n_filt
         IF (magtyp.eq.2) amagfit(k)=-99.D0
         amagfit(k)=99.D0
      END DO
      dif_w_min=1.D12
      chi2min=1.D12
      n_fil_w=1
c search for nearest filter at zs 
      DO k=1,n_filt
         dif_w=ABS(wl(k)-wl(ifilt)*(zs+1.D0))
         IF ((magtyp.eq.0.or.magtyp.eq.1).and.
     .        amag(k).gt.1.D0.and.amag(k).lt.90.D0
     .        .and.i_lyman(k).lt.1) THEN
            IF (dif_w.lt.dif_w_min) THEN   
               dif_w_min=dif_w 
               n_fil_w=k  
            END IF
         ELSE IF (magtyp.eq.2.and.amag(k).ge.0.D0
     .           .and.err_p(k).ge.0.D0) THEN
            IF (dif_w.lt.dif_w_min) THEN   
               dif_w_min=dif_w 
               n_fil_w=k  
            END IF
         END IF
      END DO
      i_typ=1
      i_age=2
      i_red=1
c -----------------------------------------------------------------------
      DO itype=1,n_files        !Scan types
         IF (sed_abc(itype).eq.'BC') THEN
            iagemin=2
            iagemax=ks(itype)
c if age_min and age_max are not in the range of values considered by BC SEDs
c and iage_check=0 then iagemin=2  and iagemax=ks(itype)
            DO iage=2,ks(itype)
               IF (age_min.gt.(t_bc(iage-1)*1.D-9).and.
     .              age_min.le.(t_bc(iage)*1.D-9)) iagemin=iage
               IF (age_max.gt.(t_bc(iage-1)*1.D-9).and.
     .              age_max.le.(t_bc(iage)*1.D-9)) iagemax=iage
            END DO
c free age between age_min and age(z)
            IF (iage_check.eq.1) THEN 
               DO iage=2,ks(itype)
                  IF (ageuni-age_form.gt.(t_bc(iage-1)*1.D-9).and.
     .                 ageuni-age_form.le.(t_bc(iage)*1.D-9).and.
c choose the minimum between age(z) and age_max
     .                 ageuni-age_form.le.t_bc(iagemax)*1.D-9) 
     .                 iagemax=iage
               END DO
c only the two ages around the age(z)
            ELSE IF (iage_check.eq.2) THEN  
               iagemin=2
               iagemax=3
               DO iage=3,ks(itype)
                  IF (ageuni-age_form.gt.(t_bc(iage-1)*1.D-9).and.
     .                 ageuni-age_form.le.(t_bc(iage)*1.D-9)) THEN
                     iagemin=iage-1
                     iagemax=iage
                  END IF
               END DO
            END IF
         ELSE IF (sed_abc(itype).eq.'PG') THEN
            IF (t_pg(1).lt.1.E-8) THEN ! skip age=0
               iagemin=2
            ELSE
               iagemin=1
            END IF
            iagemax=ks(itype)
            DO iage=iagemin,ks(itype)
               IF (age_min.gt.(t_pg(iage-1)*1.D-3).and.
     .              age_min.le.(t_pg(iage)*1.D-3)) iagemin=iage
               IF (age_max.gt.(t_pg(iage-1)*1.D-3).and.
     .              age_max.le.(t_pg(iage)*1.D-3)) iagemax=iage
            END DO
c free age between age_min and age(z)
            IF (iage_check.eq.1) THEN 
               DO iage=iagemin,ks(itype)
                  IF (ageuni-age_form.gt.(t_pg(iage-1)*1.D-3).and.
     .                 ageuni-age_form.le.(t_pg(iage)*1.D-3).and.
c choose the minimum between age(z) and age_max
     .                 ageuni-age_form.le.t_pg(iagemax)*1.D-3) 
     .                 iagemax=iage
               END DO
c only the two ages around the age(z)
            ELSE IF (iage_check.eq.2) THEN  
               IF (t_pg(1).lt.1.E-8) THEN ! skip age=0
                  iagemin=2
               ELSE
                  iagemin=1
               END IF
               iagemax=iagemin+1
               DO iage=iagemin+1,ks(itype)
                  IF (ageuni-age_form.gt.(t_pg(iage-1)*1.D-3).and.
     .                 ageuni-age_form.le.(t_pg(iage)*1.D-3)) THEN
                     iagemin=iage-1
                     iagemax=iage
                  END IF
               END DO
            END IF
         ELSE IF (sed_abc(itype).eq.'AS'.or.sed_abc(itype).eq.'A ') THEN
            iagemin=1
            iagemax=1
         END IF
c -----------------------------------------------------------------------
         DO iage=iagemin,iagemax ! Scan ages
            IF (sed_abc(itype).eq.'BC') THEN
               DO k=1,iw(itype)
                  y(k)=f_bc(k,iage,ntyp_bc(itype))
                  ww(k)=DBLE(w_bc(k,ntyp_bc(itype)))
               END DO
            ELSE IF (sed_abc(itype).eq.'PG') THEN
               DO k=1,iw(itype)
                  y(k)=f_pg(k,iage,ntyp_pg(itype))
                  ww(k)=DBLE(w_pg(k,ntyp_pg(itype)))
               END DO
            ELSE IF (sed_abc(itype).eq.'AS'.or.
     .              sed_abc(itype).eq.'A ') THEN
               DO k=1,iw(itype)
                  y(k)=f_as(k,iage,ntyp_as(itype))
                  ww(k)=DBLE(w_as(k,ntyp_as(itype)))
               END DO
            END IF
            IF (sed_abc(itype).eq.'BC') THEN
               age=t_bc(iage)*1.D-9
            ELSE IF (sed_abc(itype).eq.'AS'.or.
     .              sed_abc(itype).eq.'A ') THEN
               age=19.9999D0
            ELSE IF (sed_abc(itype).eq.'PG') THEN
               age=t_pg(iage)*1.D-3
            END IF
c -----------------------------------------------------------------------
            DO ired=1,nred      ! Scan reddening
c -----------------------------------------------------------------------
               av=avmin+(ired-1)*averr
               DO k=1,iw(itype)
                  yy(k)=y(k)
               END DO
               IF (av.gt.0.0) THEN
                  CALL red_spec(ww,yy,av,yob,1,iw(itype),law_rd) 
                  DO k=1,iw(itype)
                     yy(k)=yob(k)
                  END DO
               END IF

c ..... Lyman alpha forest
               CALL lyman_forest(ww,yy,iw(itype),zs)
c ..... Compute fluxes in broad band filters
               flux0=filter(ifilt,ww,yy,iw(itype),0.D0)
               DO k=1,n_filt+1
                  flux(k,iage,itype,ired)=
     .                 filter(k,ww,yy,iw(itype),zs)
                  IF (magtyp.eq.0) THEN 
                     fluxn(k)=flux(k,iage,itype,ired)/surf(k)
                  ELSE IF (magtyp.eq.1.or.magtyp.eq.2) THEN
                     fluxn(k)=flux(k,iage,itype,ired)/f2(k)
                  END IF
               END DO
c ..... Probability for zs, iage, itype, ired
               sxx=0.D0
               sxy=0.D0
               b=0.D0
               DO k=1,n_filt
                  IF (fl(k).ge.1.D-100) THEN ! no oof objects
                     IF (flux(k,iage,itype,ired).lt.1.D-100
     .                    .and.i_lyman(k).eq.1) THEN
                        fl1(k)=0.D0
                     ELSE
                        fl1(k)=fl(k)
                        wt=1.D0/(err_fl(k)**2)
                        sxx=sxx+fluxn(k)*fluxn(k)*wt
                        sxy=sxy+fl1(k)*wt*fluxn(k)
                     END IF
                  ELSE
                     fl1(k)=-1.D0
                  END IF
               END DO
               b=sxy/sxx
               chi2=0.D0
               DO k=1,n_filt
                  IF (fl1(k).ge.0.D0) THEN ! no oof obj.
                     chi2=chi2+((fl1(k)-b*fluxn(k))/err_fl(k))**2
                  END IF
               END DO
c write matrix               
               IF (matrix.gt.0) THEN
                  r_chi2=chi2/deg ! reduced chi2
                  qqq=100.D0*qchi2(0.5D0*deg,0.5D0*chi2)
c Absolute magnitude:
c 0. use the nearest observed filter to minimize k-correction
c 1. compute directly from the SED best fit 
                  amagb=0.0D0
                  IF (imabs.eq.0) THEN ! use the nearest rest frame filter 
                     amagb=99.
                     IF (flux(n_fil_w,iage,itype,ired).gt.0.D0) THEN
                        IF (magtyp.eq.0) THEN
                           amagb=amag1(n_fil_w)-dmod-
     .                          2.5D0*DLOG10((flux0/vega(ifilt))*
     .                          (vega(n_fil_w)/
     .                          flux(n_fil_w,iage,itype,ired)))
                        ELSE ! AB magnitude
                           amagb=amag1(n_fil_w)-dmod-
     .                          2.5D0*DLOG10((flux0/f2(ifilt))*
     .                          (f2(n_fil_w)/
     .                          flux(n_fil_w,iage,itype,ired)))
                        END IF
                     ELSE 
c use the best fit if it is impossible to compute it from the nearest filter
                        IF (magtyp.eq.0) THEN
                           amagb=-2.5D0*DLOG10(b*flux0/vega(ifilt))
     .                          -dmod
                        ELSE
                           amagb=-2.5D0*DLOG10(b*flux0/f2(ifilt))
     .                          -dmod-48.6
                        END IF
                     END IF
                  ELSE IF (imabs.eq.1) THEN ! use the best fit normalization
                     IF (magtyp.eq.0) THEN
                        amagb=-2.5D0*DLOG10(b*flux0/vega(ifilt))
     .                       -dmod
                     ELSE
                        amagb=-2.5D0*DLOG10(b*flux0/f2(ifilt))
     .                       -dmod-48.6
                     END IF
                  END IF
                  WRITE(11,2012) id,zs,itype,iage,age,av,r_chi2,
     .                 qqq,b,amagb
               END IF
c Check limiting magnitude if law_nd=4: 
c reject solution if flux template > limiting flux
               icheck=0
               DO k=1,n_filt
                  IF (fl1(k).lt.0.D0.and.law_nd(k).eq.4.and.
     .                 b*fluxn(k).gt.fl_nd(k)) THEN 
                     icheck=icheck+1                     
                  END IF
               END DO
c New minimum chi2
               IF (chi2-chi2min.lt.0.D0.and.icheck.eq.0) THEN
c Compute absolute magnitude (if not already calculated) 
                  IF (matrix.le.0) THEN
                     amagb=0.0D0
                     IF (imabs.eq.0) THEN ! use the nearest rest frame filter 
                        amagb=99.
                        IF (flux(n_fil_w,iage,itype,ired).gt.0.D0) THEN
                           IF (magtyp.eq.0) THEN
                              amagb=amag1(n_fil_w)-dmod-
     .                             2.5D0*DLOG10((flux0/vega(ifilt))*
     .                             (vega(n_fil_w)/
     .                             flux(n_fil_w,iage,itype,ired)))
                           ELSE ! AB magnitude
                              amagb=amag1(n_fil_w)-dmod-
     .                             2.5D0*DLOG10((flux0/f2(ifilt))*
     .                             (f2(n_fil_w)/
     .                             flux(n_fil_w,iage,itype,ired)))
                           END IF
                        ELSE 
c use the best fit if it is impossible to compute it from the nearest filter
                           IF (magtyp.eq.0) THEN
                              amagb=-2.5D0*DLOG10(b*flux0/vega(ifilt))
     .                             -dmod
                           ELSE
                              amagb=-2.5D0*DLOG10(b*flux0/f2(ifilt))
     .                             -dmod-48.6
                           END IF
                        END IF
                     ELSE IF (imabs.eq.1) THEN ! use the best fit normalization
                        IF (magtyp.eq.0) THEN
                           amagb=-2.5D0*DLOG10(b*flux0/vega(ifilt))
     .                          -dmod
                        ELSE
                           amagb=-2.5D0*DLOG10(b*flux0/f2(ifilt))
     .                          -dmod-48.6
                        END IF
                     END IF
                  END IF
c Check absolute magnitude: if amagb not in [amabs_1,amabs_2] reject solution
                  IF (amagb.ge.amabs_1.and.amagb.le.amabs_2) THEN
                     chi2min=chi2
                     bb=b
                     i_typ=itype
                     i_age=iage
                     i_red=ired
                     amagbbf=amagb
                  END IF
               END IF         
            END DO
         END DO
      END DO
      CLOSE(11)                 !close output matrix

c -----------------------------------------------------------------------
      DO k=1,iw(i_typ)
         yy(k)=f_pg(k,i_age,ntyp_pg(i_typ))
         ww(k)=DBLE(w_pg(k,ntyp_pg(i_typ)))
      END DO
c ..... Lyman alpha forest
      CALL lyman_forest(ww,yy,iw(i_typ),zs)
      r_chi2=chi2min/deg        ! reduced chi2
      qq=100.D0*qchi2(0.5D0*deg,0.5D0*chi2min)
      
c..... File .zobs_sed
      IF (csed1.eq.'y'.or.csed1.eq.'Y') THEN
c +++++ output in erg/cm2/s/A
         IF (iunit_sed.eq.0) THEN
c ----- input Vega magnitudes; fl(k) already in erg/cm2/s/A
            IF (magtyp.eq.0) THEN 
               DO k=1,n_filt
c detected and undetected filters and errors
                  fluxn(k)=fl(k)
                  fluxerrn(k)=err_fl(k)
c keep the negative sign of err_p for IR or UV not used filters
                  fluxn(k)=sign(fluxn(k),err_p(k))
c non detection/not observed
                  IF ((rmag(k).ge.90..and.law_nd(k).eq.0).or.
     &                 rerr_p(k).lt.0..or.rmag(k).le.0.) THEN
                     fluxn(k)=-1.D0
                     fluxerrn(k)=0.D0
c non detection with constraint on upper limit
                  ELSE IF (rmag(k).gt.90..and.law_nd(k).eq.4) THEN
                     fluxn(k)=fl_nd(k)
                     fluxerrn(k)=-1.D0                        
                  END IF
               END DO
c ----- input AB magnitudes; fl(k) in erg/cm2/s/Hz
            ELSE IF (magtyp.eq.1) THEN
               DO k=1,n_filt
                  fluxn(k)=fl(k)*f2(k)/surf(k)
                  fluxerrn(k)=err_fl(k)*f2(k)/surf(k)
c keep the negative sign of err_p for IR or UV not used filters
                  fluxn(k)=sign(fluxn(k),err_p(k))
c non detection/not observed
                  IF ((rmag(k).ge.90..and.law_nd(k).eq.0).or.
     &                 rerr_p(k).lt.0..or.rmag(k).le.0.) THEN
                     fluxn(k)=-1.D0
                     fluxerrn(k)=0.D0
c non detection with constraint on upper limit
                  ELSE IF (rmag(k).gt.90..and.law_nd(k).eq.4) THEN
                     fluxn(k)=fl_nd(k)*f2(k)/surf(k)
                     fluxerrn(k)=-1.D0                        
                  END IF
               END DO
c ----- input muJy; fl(k) in erg/cm2/s/Hz
            ELSE IF (magtyp.eq.2) THEN
               DO k=1,n_filt
                  fluxn(k)=fl(k)*f2(k)/surf(k)
                  fluxerrn(k)=err_fl(k)*f2(k)/surf(k)
c keep the negative sign of err_p for IR or UV not used filters
                  fluxn(k)=sign(fluxn(k),err_p(k))
c non detection/not observed
                  IF ((rmag(k).lt.-90..and.law_nd(k).eq.0).or.
     &                 rerr_p(k).lt.0..or.ABS(rmag(k)+1.D0).lt.1.D-6) 
     &                 THEN
                     fluxn(k)=-1.D0
                     fluxerrn(k)=0.D0
c non detection with constraint on upper limit
                  ELSE IF (rmag(k).lt.-90..and.law_nd(k).eq.4) THEN
                     fluxn(k)=fl_nd(k)*f2(k)/surf(k)
                     fluxerrn(k)=-1.D0                        
                  END IF
               END DO
            END IF
            WRITE(14,fmt2) id,(fluxn(k),k=1,n_filt),
     &           (fluxerrn(k),k=1,n_filt)
c +++++ output in muJy
         ELSE IF (iunit_sed.eq.1) THEN
c ----- input Vega magnitudes; fl(k) in erg/cm2/s/A
            IF (magtyp.eq.0) THEN
               DO k=1,n_filt
                  f_jy(k)=fl(k)*surf(k)/f2(k)*1.D+29
                  err_jy(k)=err_fl(k)*surf(k)/f2(k)*1.D+29
                  f_jy(k)=sign(f_jy(k),err_p(k))
                  IF ((rmag(k).ge.90..and.law_nd(k).eq.0).or.
     &                 rerr_p(k).lt.0..or.rmag(k).le.0.) THEN
                     f_jy(k)=-1.D0
                     err_jy(k)=0.D0
                  ELSE IF (rmag(k).gt.90..and.law_nd(k).eq.4) THEN
                     f_jy(k)=fl_nd(k)*surf(k)/f2(k)*1.D+29
                     err_jy(k)=-1.D0
                  END IF
               END DO
c ----- input AB magnitudes; fl(k) in erg/cm2/s/Hz
            ELSE IF (magtyp.eq.1) THEN
               DO k=1,n_filt
c detected filter and error
                  f_jy(k)=fl(k)*1.D29
                  err_jy(k)=err_fl(k)*1.D29
                  f_jy(k)=sign(f_jy(k),err_p(k))
                  IF ((rmag(k).ge.90..and.law_nd(k).eq.0).or.
     &                 rerr_p(k).lt.0..or.rmag(k).le.0.) THEN
                     f_jy(k)=-1.D0
                     err_jy(k)=0.D0
                  ELSE IF (rmag(k).gt.90..and.law_nd(k).eq.4) THEN
                     f_jy(k)=fl_nd(k)*1.D+29
                     err_jy(k)=-1.D0
                  END IF
               END DO
c ----- input muJy; fl(k) in erg/cm2/s/Hz
            ELSE IF (magtyp.eq.2) THEN
               DO k=1,n_filt
c detected & non detected filter and error; f_jy is the input
                  IF (rmag(k).gt.0..and.(rerr_p(k).gt.0..or.
     &                 rmag(k).lt.-90.)) THEN 
                     f_jy(k)=sign(f_jy(k),err_p(k))
c non detection/not observed
                  ELSE IF ((rmag(k).lt.-90..and.law_nd(k).eq.0).or.
     &                    rerr_p(k).lt.0..or.ABS(rmag(k)+1.D0).lt.1.D-6) 
     &                    THEN
                     f_jy(k)=-1.D0
                     err_jy(k)=0.D0
                  ELSE IF (rmag(k).lt.-90..and.law_nd(k).eq.4) THEN
                     f_jy(k)=fl_nd(k)*1.D+29
                     err_jy(k)=-1.D0
                  END IF
               END DO
            END IF
            WRITE(14,fmt2) id,(f_jy(i),i=1,n_filt),
     .           (err_jy(i),i=1,n_filt)
c +++++ output in AB magnitudes
         ELSE IF (iunit_sed.eq.2) THEN
c ----- input Vega magnitudes; fl(k) in erg/cm2/s/A
            IF (magtyp.eq.0) THEN
               DO k=1,n_filt
                  amag(k)=amag1(k)+conv_mag(k)
                  amag(k)=sign(amag(k),err_p(k))
                  IF (amag1(k).gt.90.D0) THEN
                     amag(k)=mag_nd(k)
                     err_p(k)=99.D0
                  ELSE IF (amag1(k).lt.1.D0) THEN
                     amag(k)=-1.D0
                     err_p(k)=-1.D0
                  END IF
               END DO
c ----- input AB magnitudes; amag1 AB magnitudes
            ELSE IF (magtyp.eq.1) THEN
               DO k=1,n_filt
                  amag(k)=sign(amag(k),err_p(k))
                  IF (amag1(k).gt.90.D0) THEN
                     amag(k)=mag_nd(k)
                     err_p(k)=99.D0
                  ELSE IF (amag1(k).lt.1.D0) THEN
                     amag(k)=-1.D0
                     err_p(k)=-1.D0
                  END IF
               END DO
c ----- input muJy; amag1 AB magnitudes
            ELSE IF (magtyp.eq.2) THEN
               DO k=1,n_filt
                  amag(k)=amag1(k)
                  err_p(k)=2.5D0/DLOG(10.D0)*err_jy(k)/f_jy(k)
                  amag(k)=sign(amag(k),err_p(k))
                  IF (rmag(k).lt.-90.) THEN
                     amag(k)=mag_nd(k)
                     err_p(k)=99.D0
                  ELSE IF (ABS(rmag(iii)+1.).lt.1.D-6) THEN
                     amag(k)=-1.D0
                     err_p(k)=-1.D0
                  END IF
               END DO
            END IF
            WRITE(14,fmt1) id,(amag(i),i=1,n_filt),
     .           (err_p(i),i=1,n_filt)
         END IF
      END IF

      IF (chi2min.gt.9.9D+11) THEN
         WRITE(*,'(a,i12)') 'WARNING: impossible to find a '//
     .        'solution within constraints for object ID = ', id
c ..... Write outputs in any case 
         r_chi2=9999.999D0
         qq=0.D0
         i_typ=0
         i_age=0
         age=99.D0
         av=0.D0
         bb=0.D0
         amagbbf=99.D0
         DO k=1,n_filt
            amagfit(k)=99.D0
         END DO
         WRITE(13,fmt) id,zs,r_chi2,qq,i_typ,i_age,age,av,bb,
     .        (amagfit(k),k=1,n_filt),amagbbf,iw_ob,dl
c..... File .ztemp_sed
         IF (csed2.eq.'y'.or.csed2.eq.'Y') THEN 
            DO k=1,n_filt
               mag_temp(k)=99.D0
               fl_temp(k)=-1.D0
            END DO
            IF (iunit_sed.eq.0.or.iunit_sed.eq.1) THEN
               WRITE(15,fmt4) id,(fl_temp(k),k=1,n_filt)
            ELSE IF (iunit_sed.eq.2) THEN
               WRITE(15,fmt3) id,(mag_temp(k),k=1,n_filt)
            END IF
         END IF
         GO TO 13400
      END IF

c ..... write solution
      IF (sed_abc(i_typ).eq.'BC') THEN
         DO k=1,iw(i_typ)
            y(k)=f_bc(k,i_age,ntyp_bc(i_typ))
            ww(k)=DBLE(w_bc(k,ntyp_bc(i_typ)))
         END DO
      ELSE IF (sed_abc(i_typ).eq.'PG') THEN
         DO k=1,iw(i_typ)
            y(k)=f_pg(k,i_age,ntyp_pg(i_typ))
            ww(k)=DBLE(w_pg(k,ntyp_pg(i_typ)))
         END DO
      ELSE IF (sed_abc(i_typ).eq.'AS'.or.
     .        sed_abc(i_typ).eq.'A ') THEN
         DO k=1,iw(i_typ)
            y(k)=f_as(k,i_age,ntyp_as(i_typ))
            ww(k)=DBLE(w_as(k,ntyp_as(i_typ)))
         END DO
      END IF

      av=avmin+(i_red-1)*averr
      IF (av.gt.0.0) THEN
         CALL red_spec(ww,y,av,yob,1,iw(i_typ),law_rd) 
         DO k=1,iw(i_typ)
            y(k)=yob(k)
         END DO
      END IF

      CALL lyman_forest(ww,y,iw(i_typ),zs)

      DO k=1,n_filt
         fluxfit(k)=filter(k,ww,y,iw(i_typ),zs)
         IF (fluxfit(k).gt.0.D0) THEN
            IF (magtyp.eq.0) THEN
               amagfit(k)=-2.5D0*DLOG10(bb*fluxfit(k)/vega(k))
            ELSE IF (magtyp.eq.1) THEN
               amagfit(k)=-2.5D0*DLOG10(bb*fluxfit(k)/f2(k))-48.6D0
            ELSE IF (magtyp.eq.2) THEN
               amagfit(k)=bb*fluxfit(k)/f2(k)*1.D29
            END IF
         ELSE
            amagfit(k)=99.D0
            IF (magtyp.eq.2) amagfit(k)=-99.D0
         END IF
      END DO

      IF (sed_abc(i_typ).eq.'BC') THEN
         age=t_bc(i_age)*1.D-9
      ELSE IF (sed_abc(i_typ).eq.'AS'.or.
     .        sed_abc(i_typ).eq.'A ') THEN
         age=19.9999D0
      ELSE IF (sed_abc(i_typ).eq.'PG') THEN
         age=t_pg(i_age)*1.D-3
      END IF

      IF (r_chi2.gt.9999.999D0) r_chi2=9999.999D0
      WRITE(13,fmt) id,zs,r_chi2,qq,i_typ,i_age,age,av,bb,
     .     (amagfit(k),k=1,n_filt),amagbbf,iw_ob,dl

c file .ztemp_sed
      IF (csed2.eq.'y'.or.csed2.eq.'Y') THEN 
         DO k=1,n_filt
            fl_temp(k)=filter(k,ww,y,iw(i_typ),zs)
         END DO
         IF (iunit_sed.eq.0) THEN
            WRITE(15,fmt4) id,(bb*fl_temp(k)/surf(k),k=1,n_filt)
         ELSE IF (iunit_sed.eq.1) THEN
            WRITE(15,fmt4) id,(bb*fl_temp(k)/f2(k)*1.D+29,
     &           k=1,n_filt)
         ELSE IF (iunit_sed.eq.2) THEN
            DO k=1,n_filt
               IF (fl_temp(k).gt.0.D0) THEN
                  mag_temp(k)=-2.5D0*DLOG10(bb*fl_temp(k)/f2(k))-48.60
               ELSE
                  mag_temp(k)=99.D0
               END IF
            END DO
            WRITE(15,fmt3) id,(mag_temp(k),k=1,n_filt)
         END IF
      END IF
c file .zspe
      IF (iout_s.gt.1) THEN
         spectra=root(1:iroot)//'.zspe'
         IF (iunit_sed.eq.0) THEN
            DO k=1,iw(i_typ)
               y(k)=bb*y(k)/(1.D0+zs)
            END DO
         ELSE IF (iunit_sed.eq.1) THEN
            DO k=1,iw(i_typ)
               y(k)=bb*y(k)*(ww(k)**2/c)*1.D+29*
     .              (1.D0+zs)
            END DO
         ELSE IF (iunit_sed.eq.2) THEN
            DO k=1,iw(i_typ)
               y(k)=bb*y(k)*(ww(k)**2/c)*(1.D0+zs)
               IF (y(k).ne.0.D0) THEN
                  y(k)=-2.5D0*DLOG10(y(k))-48.60D0
               ELSE
                  y(k)=100.D0
               END IF
            END DO
         END IF
         iwav=0
         wmin1=1.D5
         wmax1=0.D0
         DO k=1,n_filt
            wmin0=wl(k)-0.5D0*surf(k)-1000.D0
            wmax0=wl(k)+0.5D0*surf(k)+1000.D0
            IF (wmin0.lt.wmin1) wmin1=wmin0
            IF (wmax0.gt.wmax1) wmax1=wmax0
         END DO
c Uncomment the following 2 lines and select the range 
c of lambda you prefer for the .zspe output
c         wmin1=1000.D0          ! lambda min in Angstrom
c         wmax1=100000000.D0         ! lambda max in Angstrom
         DO k=1,iw(i_typ)
            w0=ww(k)*(1.D0+zs)
            IF ((w0.ge.wmin1.and.w0.le.wmax1).or.
     .           (w0.le.wmin1.and.w0.ge.wmax1)) THEN
               iwav=iwav+1
               wsyn(iwav)=ww(k)*(1.D0+zs)
               ysyn(iwav)=y(k)
            END IF
         END DO
         OPEN(12,FILE=spectra,STATUS='unknown')
         DO k=1,iwav
            WRITE(12,2013) wsyn(k),ysyn(k)
         END DO
         CLOSE(12)
      END IF
c.......................................................................
      GO TO 13400
13500 CONTINUE
c.......................................................................
      CLOSE(16)
      CLOSE(13)
      IF (csed1.eq.'y'.or.csed1.eq.'Y') CLOSE(14)
      IF (csed2.eq.'y'.or.csed2.eq.'Y') CLOSE(15)

      time_tot=DBLE(cputime())-time_tot
      WRITE(*,'(a)') ' Done !!!'
      WRITE(*,'(1X,A,F12.3,A)') 'Total CPU time involved : ', time_tot, 
     .     ' seconds'
      WRITE(18,'(A,F12.3,A)') '#  Total CPU time involved : ',  
     .     time_tot,' seconds'
      CLOSE(18)
      STOP
 2012 FORMAT(I10,1X,F6.3,1x,I3,1x,I3,1x,F8.4,1X,F5.2,1X,ES11.5,1X,
     .     F10.5,1X,ES9.3,1X,F9.4)
 2013 FORMAT(1X,F15.4,4X,ES20.13)
 25   STOP 'Error opening synthetic spectra files'
 78   STOP 'Error opening templates file'
 79   STOP 'Error opening catalogue file'
 80   STOP 'Error opening filters file'
 81   STOP 'Error opening output files'
 82   STOP 'Error reading filters file'
      END
c----------------------------------------------------------------------
      FUNCTION func(x)
      IMPLICIT NONE
      REAL*8 func,pchi2,x,deg,prob
      EXTERNAL pchi2
      COMMON/nfree/deg,prob
c
      func=prob-pchi2(0.5D0*deg,0.5D0*x)
      RETURN
      END

c ----------------------------------------------------------------------
c ..... Luminosity distance [Mpc] 
      FUNCTION d_l(h0,om,ov,za)
      IMPLICIT NONE
      INTEGER nstep,i
      REAL*8 h0,om,ov,z,za,c,d_l,ee,s,s1,s2,dz,a,b,x
      PARAMETER (c=2.99792458D+5,nstep=1000)
      ee(x)=DSQRT(om*(1.D0+x)**3+(1.D0-om-ov)*(1.D0+x)**2+ov)
c
      IF (za.lt.1.D-5) THEN
         z=1.D-5
      ELSE
         z=za
      END IF
      IF (om.lt.0.001D0.and.ov.lt.1.D-8) THEN
         d_l=(1.D0+z)*DLOG(1.D0+z)
      ELSE IF ((om-1.D0).lt.1.D-8.and.ov.lt.1.D-8) THEN
         d_l=2.D0*(1.D0+z)*(1.D0-1.D0/DSQRT(1.D0+z))
      ELSE IF (om.lt.1.D0.and.om.gt.0.01D0.and.ov.lt.1.D-8) THEN
         d_l=(2.D0/om**2)*(om*z-(2.D0-om)*(-1.D0+DSQRT(om*z+1.D0))) ! Mattig
      ELSE
         dz=z/DBLE(FLOAT(nstep))
         s=0.D0
         d_l=0.D0
         DO i=1,nstep+1
            a=(i-1)*dz
            b=i*dz
            s1=(1.D0/ee(a))
            s2=(1.D0/ee(b))
            s=dz*(s1+s2)*0.5D0
            d_l=d_l+s
         END DO
         d_l=(1.D0+z)*d_l
      END IF
      d_l=c/h0*d_l ! luminosity distance in Mpc
      RETURN
      END
c ----------------------------------------------------------------------



