      SUBROUTINE red_spec(wl,fl,av,yob,kred,iwavl,law_rd)
c     Reads a spectrum lambda,flux=wl,fl and calculates the new one yob 
c     with internal reddening. 
c     kred=1 -> apply reddening; kred=2 -> apply dereddening
c     iwavl=number of wavelength points
c     law_rd=1 : the standard variation of absorption with wavelength 
c                is used (Allen, 1976) - MW
c     law_rd=2 : the analytic fit of Seaton (1979) curve by 
c                Fitzpatrick (1986) - MW 
c     law_rd=3 : Fitzpatrick (1986) - LMC
c     law_rd=4 : Prevot et al. (1984) and Bouchet et al. (1985) - SMC
c     law_rd=5 : Calzetti (2000) - starburst galaxies
c     law_rd=-1 : user provided table with lambda[A],k(lambda)/RV
c    
      IMPLICIT NONE
      INTEGER kred,iwavl,law_rd,mxage_bc,mxage_pg,mxage,mxtyp_bc,
     .     mxtyp_pg,mxtyp_as,mxfil,mxred,mxlyf,mxwl_bc,mxwl_pg,
     .     mxwl_as,mxwl,mxvega,ik,i,check_red,nr,mxrp,istart,iblank,
     .     mxafil,mxwf
      REAL*8 yob,alambda,wl,fl,av,ff,ff11,ff12,slope,rv,al0,ga,
     .     c1,c2,c3,c4,p11,p12,ff99,ff100,slope2,slope1,p,ala,wu,eu,
     .     ex,cc
      CHARACTER*160 filered,line
      INCLUDE 'dimensionz.dec'
      DIMENSION yob(mxwl),wl(mxwl),fl(mxwl),wu(mxrp),eu(mxrp),ex(mxrp),
     .     cc(mxrp)
      SAVE wu,eu,slope,ff11,check_red,nr
      DATA check_red/0/
      DATA wu/mxrp*0.D0/
      DATA eu/mxrp*0.D0/
      DATA ex/mxrp*0.D0/
      COMMON/redfile/filered

c ..... -1. user provided table with lambda[A],k(lambda)/RV
c          A(lambda) =  [k(lambda)/RV]*A_V 
      IF (law_rd.eq.-1) THEN
         IF (check_red.ne.1) THEN
            i=0
            OPEN(30,file=filered,status='old')
            iblank=INDEX(filered,' ')
 300        READ(30,'(a)',ERR=301,END=301) line
            IF (line(1:1).eq.'#'.or.line.eq.' ') GO TO 300
            i=i+1
            READ(line,*) wu(i),eu(i)
            GO TO 300
 301        CLOSE(30)
            WRITE(*,'(a)') ' Extinction law from file '
     &           //filered(1:iblank)
            check_red=1
            nr=i
         END IF
c
         DO i=1,nr
            ex(i)=eu(i)*av
         END DO
         DO i=1,nr-1
            cc(i)=(ex(i)-ex(i+1))/(wu(i+1)-wu(i))
         END DO
         cc(nr)=cc(nr-1)
         istart=1
         DO ik=1,iwavl
c short wavelengths estrapolation
            IF (wl(ik).le.wu(1)) THEN 
               alambda=ex(1)+(wu(1)-wl(ik))*cc(1)
c long wavelengths extrapolation
            ELSE IF (wl(ik).ge.wu(nr)) THEN 
               alambda=ex(nr)+(wu(nr)-wl(ik))*cc(nr)
            ELSE IF (wl(ik).gt.wu(1).and.wl(ik).lt.wu(nr)) THEN
               i=istart
               DO WHILE (wl(ik).ge.wu(i+1))
                  i=i+1
               END DO
               alambda=ex(i)+(wu(i)-wl(ik))*cc(i)
               istart=i
            END IF
            IF (alambda.lt.0.D0) alambda=0.D0
            IF (kred.eq.1) yob(ik)=fl(ik)*(10**(-0.4D0*alambda))
            IF (kred.eq.2) yob(ik)=fl(ik)*(10**(0.4D0*alambda))
         END DO
         RETURN

c ..... 1. Allen (1976) : wu = lambda in A, eu = k(lambda)/R, 
c          ex = A(lambda) = k(lambda) * E(B-V) = eu * A_V ; R = 3.1
      ELSE IF (law_rd.eq.1) THEN
         rv=3.1D0
         IF (check_red.ne.1) THEN
            nr=20
            wu(1)=1000.D0
            eu(1)=4.2D0
            wu(2)=1110.D0
            eu(2)=3.7D0
            wu(3)=1250.D0
            eu(3)=3.3D0
            wu(4)=1430.D0
            eu(4)=3.D0
            wu(5)=1670.D0
            eu(5)=2.7D0
            wu(6)=2000.D0
            eu(6)=2.8D0
            wu(7)=2220.D0
            eu(7)=2.9D0
            wu(8)=2500.D0
            eu(8)=2.30D0
            wu(9)=2850.D0
            eu(9)=1.97D0
            wu(10)=3330.D0
            eu(10)=1.69D0
            wu(11)=3650.D0
            eu(11)=1.58D0
            wu(12)=4000.D0
            eu(12)=1.45D0
            wu(13)=4400.D0
            eu(13)=1.32D0
            wu(14)=5000.D0
            eu(14)=1.13D0
            wu(15)=5530.D0
            eu(15)=1.00D0
            wu(16)=6700.D0
            eu(16)=0.74D0
            wu(17)=9000.D0
            eu(17)=0.46D0
            wu(18)=10000.D0
            eu(18)=0.38D0
            wu(19)=20000.D0
            eu(19)=0.11D0
            wu(20)=100000.D0
            eu(20)=0.0D0
            check_red=1
         END IF
         DO i=1,nr
            ex(i)=eu(i)*av
         END DO
         DO i=1,nr-1
            cc(i)=(ex(i)-ex(i+1))/(wu(i+1)-wu(i))
         END DO
         cc(nr)=cc(nr-1)
         istart=1
         DO ik=1,iwavl
            IF (wl(ik).le.wu(1)) THEN 
               alambda=ex(1)+(wu(1)-wl(ik))*cc(1)
            ELSE IF (wl(ik).ge.wu(nr)) THEN 
               alambda=ex(nr)+(wu(nr)-wl(ik))*cc(nr)
            ELSE IF (wl(ik).ge.wu(1).and.wl(ik).lt.wu(nr)) THEN
               i=istart
               DO WHILE (wl(ik).ge.wu(i+1))
                  i=i+1
               END DO
               alambda=ex(i)+(wu(i)-wl(ik))*cc(i)
               istart=i
            END IF
            IF (alambda.lt.0.D0) alambda=0.D0
            IF (kred.eq.1) yob(ik)=fl(ik)*(10**(-0.4D0*alambda))
            IF (kred.eq.2) yob(ik)=fl(ik)*(10**(0.4D0*alambda))
         END DO
         RETURN

c ..... 2. Seaton (1979) - Fitzpatrick fit : 
c                       for lambda < 1200 A slope from 1100-1200
c                       for lambda > 3650 A points from Allen
c                       ff = E(lambda-V)/E(B-V) = k(lambda)-R
c                       A(lambda)=A_V*(ff/R +1) ; R = 3.1

      ELSE IF (law_rd.eq.2) THEN
         rv=3.1D0
         al0=4.595D0
         ga=1.051D0
         c1=-0.38D0
         c2=0.74D0
         c3=3.96D0
         c4=0.26D0
         IF (check_red.ne.1) THEN
            nr=10
            call red(1100.D0,al0,ga,c1,c2,c3,c4,ff)
            ff11=ff
            call red(1200.D0,al0,ga,c1,c2,c3,c4,ff)
            ff12=ff
            slope=(ff12-ff11)/100.D0
            wu(1)=3650.D0
            eu(1)=1.58D0
            wu(2)=4000.D0
            eu(2)=1.45D0
            wu(3)=4400.D0
            eu(3)=1.32D0
            wu(4)=5000.D0
            eu(4)=1.13D0
            wu(5)=5530.D0
            eu(5)=1.00D0
            wu(6)=6700.D0
            eu(6)=0.74D0
            wu(7)=9000.D0
            eu(7)=0.46D0
            wu(8)=10000.D0
            eu(8)=0.38D0
            wu(9)=20000.D0
            eu(9)=0.11D0
            wu(10)=100000.D0
            eu(10)=0.0D0
            check_red=1
         END IF
         DO i=1,nr
            ex(i)=eu(i)*av
         END DO
         DO i=1,nr-1
            cc(i)=(ex(i)-ex(i+1))/(wu(i+1)-wu(i))
         END DO
         cc(nr)=cc(nr-1)
         istart=1
         DO ik=1,iwavl
            IF (wl(ik).ge.1200.D0.and.wl(ik).le.wu(1)) THEN
               call red(wl(ik),al0,ga,c1,c2,c3,c4,ff)
               alambda=av*(ff/rv+1.D0) 
            ELSE IF (wl(ik).lt.1200.D0) THEN
               ff=ff11+(wl(ik)-1100.D0)*slope
               alambda=av*(ff/rv+1.D0)
            ELSE IF (wl(ik).ge.wu(nr)) THEN 
               alambda=ex(nr-1)+(wu(nr-1)-wl(ik))*cc(nr)
            ELSE IF (wl(ik).ge.wu(1).and.wl(ik).lt.wu(nr)) THEN
               i=istart
               DO WHILE (wl(ik).ge.wu(i+1))
                  i=i+1
               END DO
               alambda=ex(i)+(wu(i)-wl(ik))*cc(i)
               istart=i
            END IF
            IF (alambda.lt.0.D0) alambda=0.D0
            IF (kred.eq.1) yob(ik)=fl(ik)*(10**(-0.4D0*alambda))
            IF (kred.eq.2) yob(ik)=fl(ik)*(10**(0.4D0*alambda))
         END DO
         RETURN
      
c ..... 3. Fitzpatrick (1986) : 
c                       for lambda < 1200 A slope derived from 1100-1200
c                       for lambda > 3330 A points from Allen
c                       ff = E(lambda-V)/E(B-V) = k(lambda)-R
c                       A(lambda) = A_V*(ff/R +1) ; R = 3.1

      ELSE IF (law_rd.eq.3) THEN
         rv=3.1D0
         al0=4.608D0
         ga=0.994D0
         c1=-0.69D0
         c2=0.89D0
         c3=2.55D0
         c4=0.50D0
         IF (check_red.ne.1) THEN
            nr=11
            call red(1100.D0,al0,ga,c1,c2,c3,c4,ff)
            ff11=ff
            call red(1200.D0,al0,ga,c1,c2,c3,c4,ff)
            ff12=ff
            slope=(ff12-ff11)/100.D0
            wu(1)=3330.D0
            eu(1)=1.682D0
            wu(2)=3650.D0
            eu(2)=1.58D0
            wu(3)=4000.D0
            eu(3)=1.45D0
            wu(4)=4400.D0
            eu(4)=1.32D0
            wu(5)=5000.D0
            eu(5)=1.13D0
            wu(6)=5530.D0
            eu(6)=1.00D0
            wu(7)=6700.D0
            eu(7)=0.74D0
            wu(8)=9000.D0
            eu(8)=0.46D0
            wu(9)=10000.D0
            eu(9)=0.38D0
            wu(10)=20000.D0
            eu(10)=0.11D0
            wu(11)=100000.D0
            eu(11)=0.0D0
            check_red=1
         END IF

         DO i=1,nr
            ex(i)=eu(i)*av
         END DO
         DO i=1,nr-1
            cc(i)=(ex(i)-ex(i+1))/(wu(i+1)-wu(i))
         END DO
         cc(nr)=cc(nr-1)
         istart=1
         DO ik=1,iwavl
            IF (wl(ik).ge.1200.D0.and.wl(ik).lt.wu(1)) THEN
               call red(wl(ik),al0,ga,c1,c2,c3,c4,ff)
               alambda=av*(ff/rv+1.D0)
            ELSE IF (wl(ik).lt.1200.D0) THEN
               ff=ff11+(wl(ik)-1100.D0)*slope
               alambda=av*(ff/rv+1.D0)
            ELSE IF (wl(ik).gt.wu(nr)) THEN
               alambda=ex(nr-1)+(wu(nr-1)-wl(ik))*cc(nr)
            ELSE IF (wl(ik).ge.wu(1).and.wl(ik).le.wu(nr)) THEN
               i=istart
               DO WHILE (wl(ik).ge.wu(i+1))
                  i=i+1
               END DO
               alambda=ex(i)+(wu(i)-wl(ik))*cc(i)
               istart=i
            END IF
            IF (alambda.lt.0.D0) alambda=0.D0
            IF (kred.eq.1) yob(ik)=fl(ik)*(10**(-0.4D0*alambda))
            IF (kred.eq.2) yob(ik)=fl(ik)*(10**(0.4D0*alambda))
         END DO
         RETURN

c ..... 4. Prevot (1984) and Bouchet (1985) :
c                        wu = lambda in A,  
c                        eu = E(lambda-V)/E(B-V) = k(lambda)-R
c                        A(lambda) = A_V*(eu/R +1)
c                        N.B. R = 2.72

      ELSE IF (law_rd.eq.4) THEN
         rv=2.72D0
         IF (check_red.ne.1) THEN
            nr=30
            wu(1)=1275.D0
            eu(1)=13.54D0
            wu(2)=1330.D0
            eu(2)=12.52D0
            wu(3)=1385.D0
            eu(3)=11.51D0
            wu(4)=1435.D0
            eu(4)=10.80D0
            wu(5)=1490.D0
            eu(5)=9.84D0
            wu(6)=1545.D0
            eu(6)=9.28D0
            wu(7)=1595.D0
            eu(7)=9.06D0
            wu(8)=1647.D0
            eu(8)=8.49D0
            wu(9)=1700.D0
            eu(9)=8.01D0
            wu(10)=1755.D0
            eu(10)=7.71D0
            wu(11)=1810.D0
            eu(11)=7.17D0
            wu(12)=1860.D0
            eu(12)=6.90D0
            wu(13)=1910.D0
            eu(13)=6.76D0
            wu(14)=2000.D0
            eu(14)=6.38D0
            wu(15)=2115.D0
            eu(15)=5.85D0
            wu(16)=2220.D0
            eu(16)=5.30D0
            wu(17)=2335.D0
            eu(17)=4.53D0
            wu(18)=2445.D0
            eu(18)=4.24D0
            wu(19)=2550.D0
            eu(19)=3.91D0
            wu(20)=2665.D0
            eu(20)=3.49D0
            wu(21)=2778.D0
            eu(21)=3.15D0
            wu(22)=2890.D0
            eu(22)=3.00D0
            wu(23)=2995.D0
            eu(23)=2.65D0
            wu(24)=3105.D0
            eu(24)=2.29D0
            wu(25)=3704.D0
            eu(25)=1.81D0
            wu(26)=4255.D0
            eu(26)=1.00D0
            wu(27)=5291.D0
            eu(27)=0.00D0
            wu(28)=12500.D0
            eu(28)=-2.02D0
            wu(29)=16500.D0
            eu(29)=-2.36D0
            wu(30)=22000.D0
            eu(30)=-2.47D0
            check_red=1
         END IF
         
         DO i=1,nr
            ex(i)=av*(eu(i)/rv+1.D0)
         END DO
         DO i=1,nr-1
            cc(i)=(ex(i)-ex(i+1))/(wu(i+1)-wu(i))
         END DO
         cc(nr)=cc(nr-1)
         istart=1
         DO ik=1,iwavl
            IF (wl(ik).le.wu(1)) THEN 
               alambda=ex(1)+(wu(1)-wl(ik))*cc(1)
            ELSE IF(wl(ik).ge.wu(nr)) THEN 
               alambda=ex(nr-1)+(wu(nr-1)-wl(ik))*cc(nr)  
            ELSE IF (wl(ik).gt.wu(1).and.wl(ik).lt.wu(nr)) THEN
               i=istart
               DO WHILE (wl(ik).ge.wu(i+1))
                  i=i+1
               END DO
               alambda=ex(i)+(wu(i)-wl(ik))*cc(i)
               istart=i
            END IF   
            IF (alambda.lt.0.D0) alambda=0.D0
            IF (kred.eq.1) yob(ik)=fl(ik)*(10**(-0.4D0*alambda))
            IF (kred.eq.2) yob(ik)=fl(ik)*(10**(0.4D0*alambda))
         END DO
         RETURN


c ..... Calzetti (2000) : 
c                   for lambda < 1200 A slope derived from 1100-1200 A
c                   for lambda > 22000A slope derived from 21900-22000 A
c                   ff = k(lambda); A(lambda) = A_V*k(lambda)/R ; R = 4.05

      ELSE IF (law_rd.eq.5) THEN
         rv=4.05D0
         p11=1.D0/0.11D0
         ff11=2.659D0*(-2.156D0+1.509D0*p11-0.198D0*p11**2
     .        +0.011D0*p11**3)+rv
         p12=1.D0/0.12D0
         ff12=2.659D0*(-2.156D0+1.509D0*p12-0.198D0*p12**2
     .        +0.011D0*p12**3)+rv
         slope1=(ff12-ff11)/100.D0
         ff99=2.659D0*(-1.857D0+1.040D0/2.19D0)+rv
         ff100=2.659D0*(-1.857D0+1.040D0/2.2D0)+rv
         slope2=(ff100-ff99)/100.D0
         DO ik=1,iwavl
            IF (wl(ik).lt.1.D-12) THEN
               IF (kred.eq.1) yob(ik)=0.D0 
               IF (kred.eq.2) yob(ik)=fl(ik) 
            ELSE
               ala=wl(ik)*1.D-4 !wavelength in microns
               p=1.D0/ala
               IF (ala.ge.0.63D0.and.ala.le.2.2D0) THEN
                  ff=2.659D0*(-1.857D0+1.040D0*p)+rv
               ELSE IF (ala.ge.0.12D0.and.ala.lt.0.63D0) THEN
                  ff=2.659D0*(-2.156D0+1.509D0*p-0.198D0*p**2+
     .                 0.011D0*p**3)+rv
               ELSE IF (ala.lt.0.12D0) THEN
                  ff=ff11+(wl(ik)-1100.D0)*slope1
               ELSE IF (ala.gt.2.2D0) THEN
                  ff=ff99+(wl(ik)-21900.D0)*slope2
               END IF
               alambda=av*ff/rv
               IF (alambda.lt.0.D0) alambda=0.D0
               IF (kred.eq.1) yob(ik)=fl(ik)*(10**(-0.4D0*alambda))
               IF (kred.eq.2) yob(ik)=fl(ik)*(10**(0.4D0*alambda))
            END IF
         END DO
         RETURN
      END IF

      WRITE(*,*) ' No reddening law is given.....'
      RETURN
      END

c ----------------------------------------------------------------------
      SUBROUTINE red(alan,al0,ga,c1,c2,c3,c4,ff)
      IMPLICIT NONE
      REAL*8 ala,alan,al0,ga,c1,c2,c3,c4,ff,p,t1,t2

c.......According to Fitzpatrick, 86

      ala=alan*1.D-4           !wavelength in microns
      p=1.0D0/ala
      IF (p.lt.5.9D0) c4=0.0D0
      t1=c3/(((p-(al0**2/p))**2)+ga*ga)
      t2=c4*(0.539D0*((p-5.9D0)**2)+0.0564D0*((p-5.9D0)**3))
      ff=c1+c2*p+t1+t2
      RETURN
      END


