      FUNCTION pchi2(a,x)
      IMPLICIT NONE
      INTEGER i, iteven, ndeg
      REAL*8 pchi2, a, x, gamma, prod, err, rest, f, c, aa
      PARAMETER (err=5.D-7)
      EXTERNAL gamma

      IF (x.lt.0.D0.or.a.le.0.D0) STOP 
     .     'Bad input for incomplete gamma function P'
      ndeg=INT(a*2.D0)
      rest=DMOD(a*2.D0,2.D0)
      
      IF (rest.lt.0.001D0) THEN ! even degrees of freedom
         iteven=ndeg/2
         pchi2=0.D0
         prod=1.D0
         DO i=1,iteven
            pchi2=pchi2+prod
            prod=prod*x/DBLE(FLOAT(i))
         END DO
         pchi2=1.D0-DEXP(-x)*pchi2
      ELSE                      ! odd degrees of freedom
         IF (x.ge.70.D0) THEN
            pchi2=1.D0
         ELSE
            c=1.D0
            pchi2=1.D0
            aa=a
 1          aa=aa+1.D0
            c=c*x/aa
            pchi2=pchi2+c
            IF (c/pchi2.gt.err) GO TO 1
            f=(x**a)*DEXP(-x)/gamma(a)
            pchi2=f*pchi2
         END IF
      END IF
      RETURN
      END

c.......................................................................

      FUNCTION qchi2(a,x)
      IMPLICIT NONE
      INTEGER i, iteven, ndeg
      REAL*8 qchi2, a, x, gamma, prod, err, rest, f, c, aa
      PARAMETER (err=5.D-7)
      EXTERNAL gamma

      IF (x.lt.0.D0.or.a.le.0.D0) STOP 
     .     'Bad input for incomplete gamma function Q'
      ndeg=INT(a*2.D0)
      rest=DMOD(a*2.D0,2.D0)
      
      IF (rest.lt.0.001D0) THEN ! even degrees of freedom
         iteven=ndeg/2
         qchi2=0.D0
         prod=1.D0
         DO i=1,iteven
            qchi2=qchi2+prod
            prod=prod*x/DBLE(FLOAT(i))
         END DO
         qchi2=DEXP(-x)*qchi2
      ELSE                      ! odd degrees of freedom
         IF (x.ge.80.D0) THEN
            qchi2=0.D0
         ELSE
            c=1.D0
            qchi2=1.D0
            aa=a
 1          aa=aa+1.D0
            c=c*x/aa
            qchi2=qchi2+c
            if (c/qchi2.gt.err) GO TO 1
            f=(x**a)*DEXP(-x)/gamma(a)
            qchi2=1.D0-f*qchi2
         END IF
      END IF
      RETURN
      END

c.......................................................................

      FUNCTION gamma(x)
      IMPLICIT NONE
      INTEGER i
      REAL*8 x, gamma, xx, z
      REAL*8 coeff(6), fact
      DATA coeff/76.18009172947146D0,-86.50532032941677D0,
     &     24.01409824083091D0,-1.231739572450155D0,
     &     0.1208650973866179D-2,-0.5395239384953D-5/
      fact=1.000000000190015D0
      z=x
      DO i=1,6
         z=z+1.D0
         fact=fact+coeff(i)/z
      END DO
      xx=x+5.5D0
      gamma=xx**(x+0.5D0)*DEXP(-xx)*DSQRT(2.D0*DACOS(-1.D0))*fact
      RETURN
      END

c.......................................................................

      FUNCTION zero(func,xl,xu,err)
      IMPLICIT NONE
      REAL*8 zero, func, xl, xu, x1, x2, xnew, err, test, errp
      INTEGER i, itmax
      PARAMETER (itmax=5000)
      EXTERNAL func
      IF (func(xl)*func(xu).gt.0.D0) THEN
         STOP ' ahi! zero non bracketed ...'
      END IF
      x1=xl
      x2=xu
      DO i=1,itmax
         xnew=x2-(func(x2)*(x1-x2))/(func(x1)-func(x2))
         IF (x1+x2.ne.0.) THEN
            errp=DABS((x2-x1)/(x2+x1))*100.D0
         END IF
         test=func(x1)*func(xnew)
         IF (test.eq.0.D0) THEN
            errp=0.D0
         ELSE
            IF (test.lt.0.D0) THEN
               x2=xnew
            ELSE
               x1=xnew
            END IF
         END IF
         IF (errp.le.err) GO TO 20
      END DO
 20   zero=xnew
      RETURN
      END

c.......................................................................

      SUBROUTINE sort2(n,arr,brr)
      INTEGER n,M,NSTACK
      REAL*8 arr(n),brr(n)
      PARAMETER (M=7,NSTACK=50)
      INTEGER i,ir,j,jstack,k,l,istack(NSTACK)
      REAL*8 a,b,temp
      jstack=0
      l=1
      ir=n
1     if(ir-l.lt.M)then
        do 12 j=l+1,ir
          a=arr(j)
          b=brr(j)
          do 11 i=j-1,1,-1
            if(arr(i).le.a)goto 2
            arr(i+1)=arr(i)
            brr(i+1)=brr(i)
11        continue
          i=0
2         arr(i+1)=a
          brr(i+1)=b
12      continue
        if(jstack.eq.0)return
        ir=istack(jstack)
        l=istack(jstack-1)
        jstack=jstack-2
      else
        k=(l+ir)/2
        temp=arr(k)
        arr(k)=arr(l+1)
        arr(l+1)=temp
        temp=brr(k)
        brr(k)=brr(l+1)
        brr(l+1)=temp
        if(arr(l+1).gt.arr(ir))then
          temp=arr(l+1)
          arr(l+1)=arr(ir)
          arr(ir)=temp
          temp=brr(l+1)
          brr(l+1)=brr(ir)
          brr(ir)=temp
        endif
        if(arr(l).gt.arr(ir))then
          temp=arr(l)
          arr(l)=arr(ir)
          arr(ir)=temp
          temp=brr(l)
          brr(l)=brr(ir)
          brr(ir)=temp
        endif
        if(arr(l+1).gt.arr(l))then
          temp=arr(l+1)
          arr(l+1)=arr(l)
          arr(l)=temp
          temp=brr(l+1)
          brr(l+1)=brr(l)
          brr(l)=temp
        endif
        i=l+1
        j=ir
        a=arr(l)
        b=brr(l)
3       continue
          i=i+1
        if(arr(i).lt.a)goto 3
4       continue
          j=j-1
        if(arr(j).gt.a)goto 4
        if(j.lt.i)goto 5
        temp=arr(i)
        arr(i)=arr(j)
        arr(j)=temp
        temp=brr(i)
        brr(i)=brr(j)
        brr(j)=temp
        goto 3
5       arr(l)=arr(j)
        arr(j)=a
        brr(l)=brr(j)
        brr(j)=b
        jstack=jstack+2
        if(jstack.gt.NSTACK)pause 'NSTACK too small in sort2'
        if(ir-i+1.ge.j-l)then
          istack(jstack)=ir
          istack(jstack-1)=i
          ir=j-1
        else
          istack(jstack)=j-1
          istack(jstack-1)=l
          l=i
        endif
      endif
      goto 1
      END
C  (C) Copr. 1986-92 Numerical Recipes Software ]2#1.

c.......................................................................

      SUBROUTINE sort(n,arr)
      INTEGER n,M,NSTACK
      REAL*8 arr(n)
      PARAMETER (M=7,NSTACK=50)
      INTEGER i,ir,j,jstack,k,l,istack(NSTACK)
      REAL*8 a,temp
      jstack=0
      l=1
      ir=n
1     if(ir-l.lt.M)then
        do 12 j=l+1,ir
          a=arr(j)
          do 11 i=j-1,1,-1
            if(arr(i).le.a)goto 2
            arr(i+1)=arr(i)
11        continue
          i=0
2         arr(i+1)=a
12      continue
        if(jstack.eq.0)return
        ir=istack(jstack)
        l=istack(jstack-1)
        jstack=jstack-2
      else
        k=(l+ir)/2
        temp=arr(k)
        arr(k)=arr(l+1)
        arr(l+1)=temp
        if(arr(l+1).gt.arr(ir))then
          temp=arr(l+1)
          arr(l+1)=arr(ir)
          arr(ir)=temp
        endif
        if(arr(l).gt.arr(ir))then
          temp=arr(l)
          arr(l)=arr(ir)
          arr(ir)=temp
        endif
        if(arr(l+1).gt.arr(l))then
          temp=arr(l+1)
          arr(l+1)=arr(l)
          arr(l)=temp
        endif
        i=l+1
        j=ir
        a=arr(l)
3       continue
          i=i+1
        if(arr(i).lt.a)goto 3
4       continue
          j=j-1
        if(arr(j).gt.a)goto 4
        if(j.lt.i)goto 5
        temp=arr(i)
        arr(i)=arr(j)
        arr(j)=temp
        goto 3
5       arr(l)=arr(j)
        arr(j)=a
        jstack=jstack+2
        if(jstack.gt.NSTACK)pause 'NSTACK too small in sort'
        if(ir-i+1.ge.j-l)then
          istack(jstack)=ir
          istack(jstack-1)=i
          ir=j-1
        else
          istack(jstack)=j-1
          istack(jstack-1)=l
          l=i
        endif
      endif
      goto 1
      END
C  (C) Copr. 1986-92 Numerical Recipes Software ]2#1.

c.......................................................................
