c Returns age of universe [Gyr] at redshift z for a universe with 
c H0 = h0 [km/s/Mpc], Omega_matter = om, Omega_lambda = ov
c
      FUNCTION tz(h0,om,ov,z)
      IMPLICIT NONE
      INTEGER i,nstep,nstep1
      PARAMETER (nstep=10000,nstep1=50000)
      REAL*8 tz,h0,om,ov,z,dz,h_gyr,acosh,ee,x,zmid,dzi,
     .     s,s1,s2,t1,t2,a,b
      PARAMETER (zmid=50.D0)
      acosh(x)=DLOG(x+DSQRT(x**2-1.D0))
      ee(x)=DSQRT(om*(1.D0+x)**3+(1.D0-om-ov)*(1.D0+x)**2+ov)
c
      h_gyr=h0*1.022012D-3      ! 365 days/year
      IF (om.lt.1.D-6.and.ov.lt.1.D-6) THEN ! Omega_0=0 & Omega_v=0
         tz=1.D0/(1.D0+z)
      ELSE IF (ABS(om-1.D0).lt.1.D-6.and.ov.lt.1.D-6) THEN ! Omega_0=1 & Omega_v=0
         tz=(2.D0/3.D0)/(1.D0+z)**1.5D0
      ELSE IF (om.lt.1.D0.and.ov.lt.1.D-6) THEN ! Omega_0<1 & Omega_v=0
         tz=0.5D0*(om/(1.D0-om)**1.5D0)*
     .        ((2.D0/om)*DSQRT((1.D0-om)*(1.D0+om*z))/(1.D0+z)
     .        -acosh((2.D0-om*(1.D0-z))/(om*(1.D0+z))))
      ELSE IF (om.gt.1.D0.and.ov.lt.1.D-6) THEN ! Omega_0>1 & Omega_v=0
         tz=0.5D0*(om/(om-1.D0)**1.5D0)*
     .        (acos((2.D0-om*(1.D0-z))/(om*(1.D0+z)))
     .        -(2.D0/om)*DSQRT((om-1.D0)*(1.D0+om*z))/(1.D0+z))
      ELSE ! Lambda models
         IF (z.lt.1.D-5) THEN
c first step: integral in [0,zmid]
            dz=zmid/DBLE(FLOAT(nstep1))
            s=0.D0
            tz=0.D0
            t1=0.D0
            DO i=1,nstep1
               a=(i-1)*dz
               b=i*dz
               s1=1.D0/((1.D0+a)*ee(a))
               s2=1.D0/((1.D0+b)*ee(b))
               s=dz*(s1+s2)/2.D0
               t1=t1+s
            END DO
c second step: integral in [zmid,infty] with change of variable z -> 1/zi
            dzi=(1.D0/zmid)/DBLE(FLOAT(nstep))
            s=0.D0
            t2=0.D0
            DO i=2,nstep
               a=(i-1)*dzi
               b=i*dzi
               s1=1.D0/((1.D0+1.D0/a)*ee(1.D0/a)*a*a)
               s2=1.D0/((1.D0+1.D0/b)*ee(1.D0/b)*b*b)
               s=dzi*(s1+s2)*0.5D0
               t2=t2+s
            END DO
            tz=t1+t2
         ELSE
c integral in [z,infty] with change of variable z -> 1/zi
            dzi=(1.D0/z)/DBLE(FLOAT(nstep1))
            s=0.D0
            tz=0.D0
            DO i=2,nstep1
               a=(i-1)*dzi
               b=i*dzi
               s1=1.D0/((1.D0+1.D0/a)*ee(1.D0/a)*a*a)
               s2=1.D0/((1.D0+1.D0/b)*ee(1.D0/b)*b*b)
               s=dzi*(s1+s2)*0.5D0
               tz=tz+s
            END DO
         END IF
      END IF
      tz=tz/h_gyr
      RETURN
      END

